@@ -22,22 +22,27 @@
  *
  */
 
+#include <inttypes.h>
 #include <sys/stat.h>
 #include <sys/mount.h>
 #include <errno.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <limits.h>
 #include <string.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <i915_drm.h>
 
 #include "drmtest.h"
-#include "igt_display.h"
+#include "igt_kms.h"
 #include "igt_debugfs.h"
 
 /**
  * SECTION:igt_debugfs
  * @short_description: Support code for debugfs features
  * @title: i-g-t debugfs
+ * @include: igt_debugfs.h
  *
  * This library provides helpers to access debugfs features. On top of some
  * basic functions to access debugfs files with e.g. igt_debugfs_open() it also
@@ -58,7 +63,7 @@
  * another either for equality or difference. Otherwise CRCs must be treated as
  * completely opaque values. Note that not even CRCs from different pipes or tap
  * points on the same platform can be compared. Hence only use igt_crc_is_null()
- * and igt_crc_equal() to insepct CRC values captured by the same
+ * and igt_crc_equal() to inspect CRC values captured by the same
  * #igt_pipe_crc_t object.
  *
  * # Other debugfs interface wrappers
@@ -76,13 +81,12 @@
  * General debugfs helpers
  */
 
-/**
- * igt_debugfs_init:
- * @debugfs: debugfs access structure to initialize
- *
- * Initializes the debugfs access helper library.
- */
-void igt_debugfs_init(igt_debugfs_t *debugfs)
+typedef struct {
+	char root[128];
+	char dri_path[128];
+} igt_debugfs_t;
+
+static bool __igt_debugfs_init(igt_debugfs_t *debugfs)
 {
 	const char *path = "/sys/kernel/debug";
 	struct stat st;
@@ -107,27 +111,49 @@ find_minor:
 		sprintf(debugfs->dri_path + len, "/i915_error_state");
 		if (stat(debugfs->dri_path, &st) == 0) {
 			debugfs->dri_path[len] = '\0';
-			return;
+			return true;
 		}
 	}
 
 	debugfs->dri_path[0] = '\0';
 
-	igt_fail(4);
+	return false;
+}
+
+static igt_debugfs_t *__igt_debugfs_singleton(void)
+{
+	static igt_debugfs_t singleton;
+	static bool init_done = false;
+
+	if (init_done)
+		return &singleton;
+
+	if (__igt_debugfs_init(&singleton)) {
+		init_done = true;
+		return &singleton;
+	} else {
+		return NULL;
+	}
 }
 
 /**
  * igt_debugfs_open:
- * @debugfs: debugfs access structure
  * @filename: name of the debugfs node to open
  * @mode: mode bits as used by open()
  *
  * This opens a debugfs file as a Unix file descriptor. The filename should be
  * relative to the drm device's root, i.e without "drm/&lt;minor&gt;".
+ *
+ * Returns:
+ * The Unix file descriptor for the debugfs file or -1 if that didn't work out.
  */
-int igt_debugfs_open(igt_debugfs_t *debugfs, const char *filename, int mode)
+int igt_debugfs_open(const char *filename, int mode)
 {
 	char buf[1024];
+	igt_debugfs_t *debugfs = __igt_debugfs_singleton();
+
+	if (!debugfs)
+		return -1;
 
 	sprintf(buf, "%s/%s", debugfs->dri_path, filename);
 	return open(buf, mode);
@@ -135,18 +161,25 @@ int igt_debugfs_open(igt_debugfs_t *debugfs, const char *filename, int mode)
 
 /**
  * igt_debugfs_fopen:
- * @debugfs: debugfs access structure
  * @filename: name of the debugfs node to open
  * @mode: mode string as used by fopen()
  *
  * This opens a debugfs file as a libc FILE. The filename should be
  * relative to the drm device's root, i.e without "drm/&lt;minor&gt;".
+ *
+ * Returns:
+ * The libc FILE pointer for the debugfs file or NULL if that didn't work out.
  */
-FILE *igt_debugfs_fopen(igt_debugfs_t *debugfs, const char *filename,
+FILE *igt_debugfs_fopen(const char *filename,
 			const char *mode)
 {
 	char buf[1024];
 
+	igt_debugfs_t *debugfs = __igt_debugfs_singleton();
+
+	if (!debugfs)
+		return NULL;
+
 	sprintf(buf, "%s/%s", debugfs->dri_path, filename);
 	return fopen(buf, mode);
 }
@@ -276,11 +309,9 @@ static void igt_pipe_crc_pipe_off(int fd, enum pipe pipe)
 
 static void igt_pipe_crc_reset(void)
 {
-	igt_debugfs_t debugfs;
 	int fd;
 
-	igt_debugfs_init(&debugfs);
-	fd = igt_debugfs_open(&debugfs, "i915_display_crc_ctl", O_WRONLY);
+	fd = igt_debugfs_open("i915_display_crc_ctl", O_WRONLY);
 
 	igt_pipe_crc_pipe_off(fd, PIPE_A);
 	igt_pipe_crc_pipe_off(fd, PIPE_B);
@@ -295,21 +326,20 @@ static void pipe_crc_exit_handler(int sig)
 }
 
 /**
- * igt_pipe_crc_check:
- * @debugfs: debugfs access structure
+ * igt_require_pipe_crc:
  *
  * Convenience helper to check whether pipe CRC capturing is supported by the
  * kernel. Uses igt_skip to automatically skip the test/subtest if this isn't
  * the case.
  */
-void igt_pipe_crc_check(igt_debugfs_t *debugfs)
+void igt_require_pipe_crc(void)
 {
 	const char *cmd = "pipe A none";
 	FILE *ctl;
 	size_t written;
 	int ret;
 
-	ctl = igt_debugfs_fopen(debugfs, "i915_display_crc_ctl", "r+");
+	ctl = igt_debugfs_fopen("i915_display_crc_ctl", "r+");
 	igt_require_f(ctl,
 		      "No display_crc_ctl found, kernel too old\n");
 	written = fwrite(cmd, 1, strlen(cmd), ctl);
@@ -322,7 +352,6 @@ void igt_pipe_crc_check(igt_debugfs_t *debugfs)
 
 /**
  * igt_pipe_crc_new:
- * @debugfs: debugfs access structure
  * @pipe: display pipe to use as source
  * @source: CRC tap point to use as source
  *
@@ -333,8 +362,7 @@ void igt_pipe_crc_check(igt_debugfs_t *debugfs)
  * specific pipe CRC source to function properly.
  */
 igt_pipe_crc_t *
-igt_pipe_crc_new(igt_debugfs_t *debugfs, enum pipe pipe,
-		 enum intel_pipe_crc_source source)
+igt_pipe_crc_new(enum pipe pipe, enum intel_pipe_crc_source source)
 {
 	igt_pipe_crc_t *pipe_crc;
 	char buf[128];
@@ -343,12 +371,11 @@ igt_pipe_crc_new(igt_debugfs_t *debugfs, enum pipe pipe,
 
 	pipe_crc = calloc(1, sizeof(struct _igt_pipe_crc));
 
-	pipe_crc->ctl_fd = igt_debugfs_open(debugfs,
-					    "i915_display_crc_ctl", O_WRONLY);
+	pipe_crc->ctl_fd = igt_debugfs_open("i915_display_crc_ctl", O_WRONLY);
 	igt_assert(pipe_crc->ctl_fd != -1);
 
 	sprintf(buf, "i915_pipe_%c_crc", pipe_name(pipe));
-	pipe_crc->crc_fd = igt_debugfs_open(debugfs, buf, O_RDONLY);
+	pipe_crc->crc_fd = igt_debugfs_open(buf, O_RDONLY);
 	igt_assert(pipe_crc->crc_fd != -1);
 
 	pipe_crc->line_len = PIPE_CRC_LINE_LEN;
@@ -432,7 +459,10 @@ static bool read_one_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out)
 	ssize_t bytes_read;
 	char buf[pipe_crc->buffer_len];
 
+	igt_set_timeout(5);
 	bytes_read = read(pipe_crc->crc_fd, &buf, pipe_crc->line_len);
+	igt_set_timeout(0);
+
 	igt_assert_cmpint(bytes_read, ==, pipe_crc->line_len);
 	buf[bytes_read] = '\0';
 
@@ -507,15 +537,13 @@ void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc)
  */
 void igt_drop_caches_set(uint64_t val)
 {
-	igt_debugfs_t debugfs;
 	int fd;
 	char data[19];
 	size_t nbytes;
 
 	sprintf(data, "0x%" PRIx64, val);
 
-	igt_debugfs_init(&debugfs);
-	fd = igt_debugfs_open(&debugfs, "i915_gem_drop_caches", O_WRONLY);
+	fd = igt_debugfs_open("i915_gem_drop_caches", O_WRONLY);
 
 	igt_assert(fd >= 0);
 	nbytes = write(fd, data, strlen(data) + 1);
@@ -578,3 +606,123 @@ void igt_enable_prefault(void)
 {
 	igt_prefault_control(true);
 }
+
+/**
+ * igt_open_forcewake_handle:
+ *
+ * This functions opens the debugfs forcewake file and so prevents the GT from
+ * suspending. The reference is automatically dropped when the is closed.
+ *
+ * Returns:
+ * The file descriptor of the forcewake handle or -1 if that didn't work out.
+ */
+int igt_open_forcewake_handle(void)
+{
+	return igt_debugfs_open("i915_forcewake_user", O_WRONLY);
+}
+
+/**
+ * igt_to_stop_ring_flag:
+ * @ring: the specified ring flag from execbuf ioctl (I915_EXEC_*)
+ *
+ * This converts the specified ring to a ring flag to be used
+ * with igt_get_stop_rings() and igt_set_stop_rings().
+ *
+ * Returns:
+ * Ring flag for the given ring.
+ */
+enum stop_ring_flags igt_to_stop_ring_flag(int ring) {
+	if (ring == I915_EXEC_DEFAULT)
+		return STOP_RING_RENDER;
+
+	igt_assert(ring && ((ring & ~I915_EXEC_RING_MASK) == 0));
+	return 1 << (ring - 1);
+}
+
+static void stop_rings_write(uint32_t mask)
+{
+	int fd;
+	char buf[80];
+
+	igt_assert(snprintf(buf, sizeof(buf), "0x%08x", mask) == 10);
+	fd = igt_debugfs_open("i915_ring_stop", O_WRONLY);
+	igt_assert(fd >= 0);
+
+	igt_assert(write(fd, buf, strlen(buf)) == strlen(buf));
+	close(fd);
+}
+
+/**
+ * igt_get_stop_rings:
+ *
+ * Read current ring flags from 'i915_ring_stop' debugfs entry.
+ *
+ * Returns:
+ * Current ring flags.
+ */
+enum stop_ring_flags igt_get_stop_rings(void)
+{
+	int fd;
+	char buf[80];
+	int l;
+	unsigned long long ring_mask;
+
+	fd = igt_debugfs_open("i915_ring_stop", O_RDONLY);
+	igt_assert(fd >= 0);
+	l = read(fd, buf, sizeof(buf)-1);
+	igt_assert(l > 0);
+	igt_assert(l < sizeof(buf));
+
+	buf[l] = '\0';
+
+	close(fd);
+
+	errno = 0;
+	ring_mask = strtoull(buf, NULL, 0);
+	igt_assert(errno == 0);
+	return ring_mask;
+}
+
+/**
+ * igt_set_stop_rings:
+ * @flags: Ring flags to write
+ *
+ * This writes @flags to 'i915_ring_stop' debugfs entry. Driver will
+ * prevent the CPU from writing tail pointer for the ring that @flags
+ * specify. Note that the ring is not stopped right away. Instead any
+ * further command emissions won't be executed after the flag is set.
+ *
+ * This is the least invasive way to make the GPU stuck. Hence you must
+ * set this after a batch submission with it's own invalid or endless
+ * looping instructions. In this case it is merely for giving notification
+ * for the driver that this was simulated hang, as the batch would have
+ * caused hang in any case. On the other hand if you use a valid or noop
+ * batch and want to hang the ring (GPU), you must set corresponding flag
+ * before submitting the batch.
+ *
+ * Driver checks periodically if a ring is making any progress, and if
+ * it is not, it will declare the ring to be hung and will reset the GPU.
+ * After reset, the driver will clear flags in 'i915_ring_stop'
+ *
+ * Note: Always when hanging the GPU, use igt_set_stop_rings() to
+ * notify the driver. Driver controls hang log messaging based on
+ * these flags and thus prevents false positives on logs.
+ */
+void igt_set_stop_rings(enum stop_ring_flags flags)
+{
+	enum stop_ring_flags current;
+
+	igt_assert((flags & ~(STOP_RING_ALL |
+			      STOP_RING_ALLOW_BAN |
+			      STOP_RING_ALLOW_ERRORS)) == 0);
+
+	current = igt_get_stop_rings();
+	igt_assert_f(flags == 0 || current == 0,
+		     "previous i915_ring_stop is still 0x%x\n", current);
+
+	stop_rings_write(flags);
+	current = igt_get_stop_rings();
+	if (current != flags)
+		igt_warn("i915_ring_stop readback mismatch 0x%x vs 0x%x\n",
+			 flags, current);
+}
