@@ -30,7 +30,23 @@
  */
 
 #include <pthread.h>
-#include "rendercopy.h"
+#include <stdlib.h>
+#include <sys/ioctl.h>
+#include <stdio.h>
+#include <string.h>
+#include <fcntl.h>
+#include <inttypes.h>
+#include <errno.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+#include <getopt.h>
+#include "drm.h"
+#include "ioctl_wrappers.h"
+#include "drmtest.h"
+#include "intel_bufmgr.h"
+#include "intel_batchbuffer.h"
+#include "intel_io.h"
+#include "intel_chipset.h"
 
 /* options */
 int num_contexts = 10;
@@ -44,7 +60,7 @@ int devid;
 int fd;
 
 static void init_buffer(drm_intel_bufmgr *bufmgr,
-			struct scratch_buf *buf,
+			struct igt_buf *buf,
 			uint32_t size)
 {
 	buf->bo = drm_intel_bo_alloc(bufmgr, "", size, 4096);
@@ -57,7 +73,7 @@ static void init_buffer(drm_intel_bufmgr *bufmgr,
 static void *work(void *arg)
 {
 	struct intel_batchbuffer *batch;
-	render_copyfunc_t rendercopy = get_render_copyfunc(devid);
+	igt_render_copyfunc_t rendercopy = igt_get_render_copyfunc(devid);
 	drm_intel_context *context;
 	drm_intel_bufmgr *bufmgr;
 	int td_fd;
@@ -76,7 +92,7 @@ static void *work(void *arg)
 	igt_require(context);
 
 	for (i = 0; i < iter; i++) {
-		struct scratch_buf src, dst;
+		struct igt_buf src, dst;
 
 		init_buffer(bufmgr, &src, 4096);
 		init_buffer(bufmgr, &dst, 4096);
