@@ -40,9 +40,12 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
-#include "drm.h"
-#include "i915_drm.h"
+
+#include <drm.h>
+
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
+#include "igt_aux.h"
 
 struct local_drm_i915_gem_context_destroy {
 	__u32 ctx_id;
@@ -56,6 +59,7 @@ static void context_destroy(int fd, uint32_t ctx_id)
 	struct local_drm_i915_gem_context_destroy destroy;
 	destroy.ctx_id = ctx_id;
 	do_ioctl(fd, CONTEXT_DESTROY_IOCTL, &destroy);
+#include "igt_aux.h"
 }
 
 /* Copied from gem_exec_nop.c */
@@ -146,10 +150,10 @@ static void big_exec(int fd, uint32_t handle, int ring)
 		gem_close(fd, gem_exec[i].handle);
 		gem_exec[i].handle = handle;
 		execbuf.buffer_count--;
-		printf("trying buffer count %i\n", i - 1);
+		igt_info("trying buffer count %i\n", i - 1);
 	}
 
-	printf("reduced buffer count to %i from %i\n",
+	igt_info("reduced buffer count to %i from %i\n",
 	       i - 1, num_buffers);
 
 	/* double check that it works */
