@@ -37,10 +37,10 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 #include "intel_chipset.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 
 #define OBJECT_SIZE 16384
 
@@ -50,7 +50,8 @@
 #define BLT_SRC_TILED		(1<<15)
 #define BLT_DST_TILED		(1<<11)
 
-static int gem_linear_blt(uint32_t *batch,
+static int gem_linear_blt(int fd,
+			  uint32_t *batch,
 			  uint32_t src,
 			  uint32_t dst,
 			  uint32_t length,
@@ -64,6 +65,8 @@ static int gem_linear_blt(uint32_t *batch,
 	if (height) {
 		int i = 0;
 		b[i++] = COPY_BLT_CMD | BLT_WRITE_ALPHA | BLT_WRITE_RGB;
+		if (intel_gen(intel_get_drm_devid(fd)) >= 8)
+			b[i-1]+=2;
 		b[i++] = 0xcc << 16 | 1 << 25 | 1 << 24 | (16*1024);
 		b[i++] = 0;
 		b[i++] = height << 16 | (4*1024);
@@ -75,17 +78,23 @@ static int gem_linear_blt(uint32_t *batch,
 		reloc->write_domain = I915_GEM_DOMAIN_RENDER;
 		reloc->presumed_offset = 0;
 		reloc++;
+		if (intel_gen(intel_get_drm_devid(fd)) >= 8)
+			b[i++] = 0; /* FIXME */
 
 		b[i++] = 0;
 		b[i++] = 16*1024;
 		b[i++] = 0;
 		reloc->offset = (b-batch+7) * sizeof(uint32_t);
+		if (intel_gen(intel_get_drm_devid(fd)) >= 8)
+			reloc->offset += sizeof(uint32_t);
 		reloc->delta = 0;
 		reloc->target_handle = src;
 		reloc->read_domains = I915_GEM_DOMAIN_RENDER;
 		reloc->write_domain = 0;
 		reloc->presumed_offset = 0;
 		reloc++;
+		if (intel_gen(intel_get_drm_devid(fd)) >= 8)
+			b[i++] = 0; /* FIXME */
 
 		b += i;
 		length -= height * 16*1024;
@@ -94,6 +103,8 @@ static int gem_linear_blt(uint32_t *batch,
 	if (length) {
 		int i = 0;
 		b[i++] = COPY_BLT_CMD | BLT_WRITE_ALPHA | BLT_WRITE_RGB;
+		if (intel_gen(intel_get_drm_devid(fd)) >= 8)
+			b[i-1]+=2;
 		b[i++] = 0xcc << 16 | 1 << 25 | 1 << 24 | (16*1024);
 		b[i++] = height << 16;
 		b[i++] = (1+height) << 16 | (length / 4);
@@ -105,17 +116,23 @@ static int gem_linear_blt(uint32_t *batch,
 		reloc->write_domain = I915_GEM_DOMAIN_RENDER;
 		reloc->presumed_offset = 0;
 		reloc++;
+		if (intel_gen(intel_get_drm_devid(fd)) >= 8)
+			b[i++] = 0; /* FIXME */
 
 		b[i++] = height << 16;
 		b[i++] = 16*1024;
 		b[i++] = 0;
 		reloc->offset = (b-batch+7) * sizeof(uint32_t);
+		if (intel_gen(intel_get_drm_devid(fd)) >= 8)
+			reloc->offset += sizeof(uint32_t);
 		reloc->delta = 0;
 		reloc->target_handle = src;
 		reloc->read_domains = I915_GEM_DOMAIN_RENDER;
 		reloc->write_domain = 0;
 		reloc->presumed_offset = 0;
 		reloc++;
+		if (intel_gen(intel_get_drm_devid(fd)) >= 8)
+			b[i++] = 0; /* FIXME */
 
 		b += i;
 	}
@@ -168,7 +185,7 @@ static void run(int object_size)
 	src = gem_create(fd, object_size);
 	dst = gem_create(fd, object_size);
 
-	len = gem_linear_blt(buf, src, dst, object_size, reloc);
+	len = gem_linear_blt(fd, buf, src, dst, object_size, reloc);
 	gem_write(fd, handle, 0, buf, len);
 
 	exec[0].handle = src;
@@ -190,7 +207,10 @@ static void run(int object_size)
 	exec[1].rsvd2 = 0;
 
 	exec[2].handle = handle;
-	exec[2].relocation_count = len > 40 ? 4 : 2;
+	if (intel_gen(intel_get_drm_devid(fd)) >= 8)
+		exec[2].relocation_count = len > 56 ? 4 : 2;
+	else
+		exec[2].relocation_count = len > 40 ? 4 : 2;
 	exec[2].relocs_ptr = (uintptr_t)reloc;
 	exec[2].alignment = 0;
 	exec[2].offset = 0;
@@ -214,7 +234,7 @@ static void run(int object_size)
 	i915_execbuffer2_set_context_id(execbuf, 0);
 	execbuf.rsvd2 = 0;
 
-	for (count = 1; count <= 1<<17; count <<= 1) {
+	for (count = 1; count <= 1<<12; count <<= 1) {
 		struct timeval start, end;
 
 		gettimeofday(&start, NULL);
@@ -222,10 +242,10 @@ static void run(int object_size)
 			gem_execbuf(fd, &execbuf);
 		gem_sync(fd, handle);
 		gettimeofday(&end, NULL);
-		printf("Time to blt %d bytes x %6d:	%7.3fµs, %s\n",
-		       object_size, count,
-		       elapsed(&start, &end, count),
-		       bytes_per_sec((char *)buf, object_size/elapsed(&start, &end, count)*1e6));
+		igt_info("Time to blt %d bytes x %6d:	%7.3fµs, %s\n",
+			 object_size, count,
+			 elapsed(&start, &end, count),
+			 bytes_per_sec((char *)buf, object_size/elapsed(&start, &end, count)*1e6));
 		fflush(stdout);
 	}
 	gem_close(fd, handle);
@@ -237,6 +257,8 @@ int main(int argc, char **argv)
 {
 	int i;
 
+	igt_simple_init();
+
 	igt_skip_on_simulation();
 
 	if (argc > 1) {
