@@ -37,7 +37,26 @@
 #include "config.h"
 #endif
 
-#include "rendercopy.h"
+#include <stdlib.h>
+#include <sys/ioctl.h>
+#include <stdio.h>
+#include <string.h>
+#include <fcntl.h>
+#include <inttypes.h>
+#include <errno.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+#include <getopt.h>
+
+#include <drm.h>
+
+#include "ioctl_wrappers.h"
+#include "drmtest.h"
+#include "intel_bufmgr.h"
+#include "intel_batchbuffer.h"
+#include "intel_io.h"
+#include "intel_chipset.h"
+#include "igt_aux.h"
 
 #define WIDTH 512
 #define STRIDE (WIDTH*4)
@@ -45,7 +64,7 @@
 #define SIZE (HEIGHT*STRIDE)
 
 static uint32_t linear[WIDTH*HEIGHT];
-static render_copyfunc_t render_copy;
+static igt_render_copyfunc_t render_copy;
 
 static void
 check_bo(int fd, uint32_t handle, uint32_t val)
@@ -54,12 +73,10 @@ check_bo(int fd, uint32_t handle, uint32_t val)
 
 	gem_read(fd, handle, 0, linear, sizeof(linear));
 	for (i = 0; i < WIDTH*HEIGHT; i++) {
-		if (linear[i] != val) {
-			fprintf(stderr, "Expected 0x%08x, found 0x%08x "
-				"at offset 0x%08x\n",
-				val, linear[i], i * 4);
-			abort();
-		}
+		igt_assert_f(linear[i] == val,
+			     "Expected 0x%08x, found 0x%08x "
+			     "at offset 0x%08x\n",
+			     val, linear[i], i * 4);
 		val++;
 	}
 }
@@ -77,11 +94,8 @@ int main(int argc, char **argv)
 
 	fd = drm_open_any();
 
-	render_copy = get_render_copyfunc(intel_get_drm_devid(fd));
-	if (render_copy == NULL) {
-		printf("no render-copy function, doing nothing\n");
-		return 77;
-	}
+	render_copy = igt_get_render_copyfunc(intel_get_drm_devid(fd));
+	igt_require(render_copy);
 
 	bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 	batch = intel_batchbuffer_alloc(bufmgr, intel_get_drm_devid(fd));
@@ -101,7 +115,7 @@ int main(int argc, char **argv)
 
 	if (count > intel_get_total_ram_mb() * 9 / 10) {
 		count = intel_get_total_ram_mb() * 9 / 10;
-		printf("not enough RAM to run test, reducing buffer count\n");
+		igt_info("not enough RAM to run test, reducing buffer count\n");
 	}
 
 	bo = malloc(sizeof(*bo)*count);
@@ -115,13 +129,13 @@ int main(int argc, char **argv)
 		gem_write(fd, bo[i]->handle, 0, linear, sizeof(linear));
 	}
 
-	printf("Verifying initialisation...\n");
+	igt_info("Verifying initialisation...\n");
 	for (i = 0; i < count; i++)
 		check_bo(fd, bo[i]->handle, start_val[i]);
 
-	printf("Cyclic blits, forward...\n");
+	igt_info("Cyclic blits, forward...\n");
 	for (i = 0; i < count * 4; i++) {
-		struct scratch_buf src, dst;
+		struct igt_buf src, dst;
 
 		src.bo = bo[i % count];
 		src.stride = STRIDE;
@@ -142,9 +156,9 @@ int main(int argc, char **argv)
 	if (igt_run_in_simulation())
 		return 0;
 
-	printf("Cyclic blits, backward...\n");
+	igt_info("Cyclic blits, backward...\n");
 	for (i = 0; i < count * 4; i++) {
-		struct scratch_buf src, dst;
+		struct igt_buf src, dst;
 
 		src.bo = bo[(i + 1) % count];
 		src.stride = STRIDE;
@@ -162,9 +176,9 @@ int main(int argc, char **argv)
 	for (i = 0; i < count; i++)
 		check_bo(fd, bo[i]->handle, start_val[i]);
 
-	printf("Random blits...\n");
+	igt_info("Random blits...\n");
 	for (i = 0; i < count * 4; i++) {
-		struct scratch_buf src, dst;
+		struct igt_buf src, dst;
 		int s = random() % count;
 		int d = random() % count;
 
