@@ -35,11 +35,12 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 #include "intel_bufmgr.h"
 #include "intel_batchbuffer.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
+#include "intel_chipset.h"
 
 static drm_intel_bufmgr *bufmgr;
 struct intel_batchbuffer *batch;
@@ -51,26 +52,42 @@ static int has_ppgtt = 0;
  */
 
 static void
-store_dword_loop(int divider)
+emit_store_dword_imm(int devid, drm_intel_bo *dest, uint32_t val)
 {
-	int cmd, i, val = 0;
-	uint32_t *buf;
-
-	printf("running storedw loop on render with stall every %i batch\n", divider);
-
+	int cmd;
 	cmd = MI_STORE_DWORD_IMM;
 	if (!has_ppgtt)
 		cmd |= MI_MEM_VIRTUAL;
 
-	for (i = 0; i < SLOW_QUICK(0x100000, 0x10); i++) {
+	if (intel_gen(devid) >= 8) {
+		BEGIN_BATCH(4);
+		OUT_BATCH(cmd);
+		OUT_RELOC(dest, I915_GEM_DOMAIN_INSTRUCTION,
+			  I915_GEM_DOMAIN_INSTRUCTION, 0);
+		OUT_BATCH(0);
+		OUT_BATCH(val);
+		ADVANCE_BATCH();
+	} else {
 		BEGIN_BATCH(4);
 		OUT_BATCH(cmd);
 		OUT_BATCH(0); /* reserved */
-		OUT_RELOC(target_buffer, I915_GEM_DOMAIN_INSTRUCTION,
+		OUT_RELOC(dest, I915_GEM_DOMAIN_INSTRUCTION,
 			  I915_GEM_DOMAIN_INSTRUCTION, 0);
 		OUT_BATCH(val);
 		ADVANCE_BATCH();
+	}
+}
 
+static void
+store_dword_loop(int devid, int divider)
+{
+	int i, val = 0;
+	uint32_t *buf;
+
+	igt_info("running storedw loop on render with stall every %i batch\n", divider);
+
+	for (i = 0; i < SLOW_QUICK(0x2000, 0x10); i++) {
+		emit_store_dword_imm(devid, target_buffer, val);
 		intel_batchbuffer_flush_on_ring(batch, 0);
 
 		if (i % divider != 0)
@@ -92,12 +109,12 @@ cont:
 	drm_intel_bo_map(target_buffer, 0);
 	buf = target_buffer->virtual;
 
-	printf("completed %d writes successfully, current value: 0x%08x\n", i,
+	igt_info("completed %d writes successfully, current value: 0x%08x\n", i,
 			buf[0]);
 	drm_intel_bo_unmap(target_buffer);
 }
 
-int main(int argc, char **argv)
+igt_simple_main
 {
 	int fd;
 	int devid;
@@ -121,11 +138,11 @@ int main(int argc, char **argv)
 	target_buffer = drm_intel_bo_alloc(bufmgr, "target bo", 4096, 4096);
 	igt_assert(target_buffer);
 
-	store_dword_loop(1);
-	store_dword_loop(2);
+	store_dword_loop(devid, 1);
+	store_dword_loop(devid, 2);
 	if (!igt_run_in_simulation()) {
-		store_dword_loop(3);
-		store_dword_loop(5);
+		store_dword_loop(devid, 3);
+		store_dword_loop(devid, 5);
 	}
 
 	drm_intel_bo_unreference(target_buffer);
@@ -133,6 +150,4 @@ int main(int argc, char **argv)
 	drm_intel_bufmgr_destroy(bufmgr);
 
 	close(fd);
-
-	return 0;
 }
