@@ -27,8 +27,26 @@
 
 #include <stdio.h>
 #include <time.h>
-#include "drm.h"
-#include "rendercopy.h"
+#include <stdlib.h>
+#include <sys/ioctl.h>
+#include <stdio.h>
+#include <string.h>
+#include <fcntl.h>
+#include <inttypes.h>
+#include <errno.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+#include <getopt.h>
+
+#include <drm.h>
+
+#include "ioctl_wrappers.h"
+#include "drmtest.h"
+#include "intel_bufmgr.h"
+#include "intel_batchbuffer.h"
+#include "intel_io.h"
+#include "intel_chipset.h"
+#include "igt_aux.h"
 
 #define MSEC_PER_SEC	1000L
 #define USEC_PER_MSEC	1000L
@@ -86,16 +104,26 @@ static void blt_color_fill(struct intel_batchbuffer *batch,
 {
 	const unsigned short height = pages/4;
 	const unsigned short width =  4096;
-	BEGIN_BATCH(5);
-	OUT_BATCH(COLOR_BLT_CMD		|
-		  COLOR_BLT_WRITE_ALPHA	|
-		  COLOR_BLT_WRITE_RGB);
+
+	if (intel_gen(batch->devid) >= 8) {
+		BEGIN_BATCH(8);
+		OUT_BATCH(MI_NOOP);
+		OUT_BATCH(XY_COLOR_BLT_CMD_NOLEN | 5 |
+			  COLOR_BLT_WRITE_ALPHA	| XY_COLOR_BLT_WRITE_RGB);
+	} else {
+		BEGIN_BATCH(6);
+		OUT_BATCH(XY_COLOR_BLT_CMD_NOLEN | 4 |
+			  COLOR_BLT_WRITE_ALPHA	| XY_COLOR_BLT_WRITE_RGB);
+	}
 	OUT_BATCH((3 << 24)	| /* 32 Bit Color */
-		  0xF0		| /* Raster OP copy background register */
+		  (0xF0 << 16)	| /* Raster OP copy background register */
 		  0);		  /* Dest pitch is 0 */
+	OUT_BATCH(0);
 	OUT_BATCH(width << 16	|
 		  height);
 	OUT_RELOC(buf, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
+	if (intel_gen(batch->devid) >= 8)
+		OUT_BATCH(0);
 	OUT_BATCH(rand()); /* random pattern */
 	ADVANCE_BATCH();
 }
@@ -153,9 +181,9 @@ igt_simple_main
 			iter <<= 1;
 	} while (!done && iter < 1000000);
 
-	igt_assert(iter < 1000000);
+	igt_assert_cmpint(iter, <, 1000000);
 
-	printf("%d iters is enough work\n", iter);
+	igt_info("%d iters is enough work\n", iter);
 	gem_quiescent_gpu(fd);
 	if (do_signals)
 		igt_fork_signal_helper();
@@ -171,11 +199,11 @@ igt_simple_main
 
 	igt_assert(gem_bo_wait_timeout(fd, dst2->handle, &timeout) == 0);
 	igt_assert(gem_bo_busy(fd, dst2->handle) == false);
-	igt_assert(timeout != 0);
+	igt_assert_cmpint(timeout, !=, 0);
 	if (timeout ==  (ENOUGH_WORK_IN_SECONDS * NSEC_PER_SEC))
-		printf("Buffer was already done!\n");
+		igt_info("Buffer was already done!\n");
 	else {
-		printf("Finished with %lu time remaining\n", timeout);
+		igt_info("Finished with %lu time remaining\n", timeout);
 	}
 
 	/* check that polling with timeout=0 works. */
