@@ -35,11 +35,16 @@
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <linux/kd.h>
+#include <time.h>
+#include <pthread.h>
 
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
-#include "testdisplay.h"
-#include "rendercopy.h"
+#include "intel_chipset.h"
+#include "intel_batchbuffer.h"
+#include "igt_kms.h"
+#include "igt_aux.h"
+#include "igt_debugfs.h"
 
 #define TEST_DPMS		(1 << 0)
 #define TEST_WITH_DUMMY_BCS	(1 << 1)
@@ -64,6 +69,13 @@
 #define TEST_NO_2X_OUTPUT	(1 << 20)
 #define TEST_DPMS_OFF_OTHERS	(1 << 21)
 #define TEST_ENOENT		(1 << 22)
+#define TEST_FENCE_STRESS	(1 << 23)
+#define TEST_VBLANK_RACE	(1 << 24)
+#define TEST_RPM		(1 << 25)
+#define TEST_SUSPEND		(1 << 26)
+#define TEST_TS_CONT		(1 << 27)
+#define TEST_BO_TOOBIG		(1 << 28)
+#define TEST_HANG_ONCE		(1 << 29)
 
 #define EVENT_FLIP		(1 << 0)
 #define EVENT_VBLANK		(1 << 1)
@@ -81,6 +93,7 @@ struct intel_batchbuffer *batch;
 uint32_t devid;
 int test_time = 3;
 static bool monotonic_timestamp;
+static pthread_t vblank_wait_thread;
 
 static drmModeConnector *last_connector;
 
@@ -131,7 +144,7 @@ struct test_output {
 	unsigned int fb_height;
 	unsigned int fb_ids[3];
 	int bpp, depth;
-	struct kmstest_fb fb_info[3];
+	struct igt_fb fb_info[3];
 
 	struct event_state flip_state;
 	struct event_state vblank_state;
@@ -155,7 +168,7 @@ static void emit_dummy_load__bcs(struct test_output *o)
 {
 	int i, limit;
 	drm_intel_bo *dummy_bo, *target_bo, *tmp_bo;
-	struct kmstest_fb *fb_info = &o->fb_info[o->current_fb_id];
+	struct igt_fb *fb_info = &o->fb_info[o->current_fb_id];
 	unsigned pitch = fb_info->stride;
 
 	limit = intel_gen(devid) < 6 ? 500 : 5000;
@@ -166,19 +179,18 @@ static void emit_dummy_load__bcs(struct test_output *o)
 	igt_assert(target_bo);
 
 	for (i = 0; i < limit; i++) {
-		BEGIN_BATCH(8);
-		OUT_BATCH(XY_SRC_COPY_BLT_CMD |
-			  XY_SRC_COPY_BLT_WRITE_ALPHA |
-			  XY_SRC_COPY_BLT_WRITE_RGB);
+		BLIT_COPY_BATCH_START(devid, 0);
 		OUT_BATCH((3 << 24) | /* 32 bits */
 			  (0xcc << 16) | /* copy ROP */
 			  pitch);
 		OUT_BATCH(0 << 16 | 0);
 		OUT_BATCH(o->fb_height << 16 | o->fb_width);
 		OUT_RELOC_FENCED(dummy_bo, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
+		BLIT_RELOC_UDW(devid);
 		OUT_BATCH(0 << 16 | 0);
 		OUT_BATCH(pitch);
 		OUT_RELOC_FENCED(target_bo, I915_GEM_DOMAIN_RENDER, 0, 0);
+		BLIT_RELOC_UDW(devid);
 		ADVANCE_BATCH();
 
 		if (IS_GEN6(devid) || IS_GEN7(devid)) {
@@ -199,14 +211,56 @@ static void emit_dummy_load__bcs(struct test_output *o)
 	drm_intel_bo_unreference(target_bo);
 }
 
+static void emit_fence_stress(struct test_output *o)
+{
+	const int num_fences = gem_available_fences(drm_fd);
+	struct igt_fb *fb_info = &o->fb_info[o->current_fb_id];
+	struct drm_i915_gem_execbuffer2 execbuf;
+	struct drm_i915_gem_exec_object2 *exec;
+	uint32_t buf[2] = { MI_BATCH_BUFFER_END, 0 };
+	drm_intel_bo **bo;
+	int i;
+
+	bo = calloc(sizeof(*bo), num_fences);
+	exec = calloc(sizeof(*exec), num_fences+1);
+	for (i = 0; i < num_fences - 1; i++) {
+		uint32_t tiling = I915_TILING_X;
+		unsigned long pitch = 0;
+		bo[i] = drm_intel_bo_alloc_tiled(bufmgr,
+						 "tiled bo", 1024, 1024, 4,
+						 &tiling, &pitch, 0);
+		exec[i].handle = bo[i]->handle;
+		exec[i].flags = EXEC_OBJECT_NEEDS_FENCE;
+	}
+	exec[i].handle = fb_info->gem_handle;
+	exec[i].flags = EXEC_OBJECT_NEEDS_FENCE;
+	exec[++i].handle = gem_create(drm_fd, 4096);
+	gem_write(drm_fd, exec[i].handle, 0, buf, sizeof(buf));
+
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = (uintptr_t)exec;
+	execbuf.buffer_count = i + 1;
+	execbuf.batch_len = sizeof(buf);
+	if (HAS_BLT_RING(intel_get_drm_devid(drm_fd)))
+		execbuf.flags = I915_EXEC_BLT;
+
+	gem_execbuf(drm_fd, &execbuf);
+
+	gem_close(drm_fd, exec[i].handle);
+	for (i = 0; i < num_fences - 1; i++)
+		drm_intel_bo_unreference(bo[i]);
+	free(bo);
+	free(exec);
+}
+
 static void emit_dummy_load__rcs(struct test_output *o)
 {
-	const struct kmstest_fb *fb_info = &o->fb_info[o->current_fb_id];
-	render_copyfunc_t copyfunc;
-	struct scratch_buf sb[2], *src, *dst;
+	const struct igt_fb *fb_info = &o->fb_info[o->current_fb_id];
+	igt_render_copyfunc_t copyfunc;
+	struct igt_buf sb[2], *src, *dst;
 	int i, limit;
 
-	copyfunc = get_render_copyfunc(devid);
+	copyfunc = igt_get_render_copyfunc(devid);
 	if (copyfunc == NULL)
 		return emit_dummy_load__bcs(o);
 
@@ -232,9 +286,9 @@ static void emit_dummy_load__rcs(struct test_output *o)
 	dst = &sb[1];
 
 	for (i = 0; i < limit; i++) {
-		struct scratch_buf *tmp;
+		struct igt_buf *tmp;
 
-		copyfunc(batch,
+		copyfunc(batch, NULL,
 			 src, 0, 0,
 			 o->fb_width, o->fb_height,
 			 dst, 0, 0);
@@ -249,34 +303,6 @@ static void emit_dummy_load__rcs(struct test_output *o)
 	drm_intel_bo_unreference(sb[1].bo);
 }
 
-static void set_connector_dpms(drmModeConnector *connector, int mode)
-{
-	int i, dpms = 0;
-	bool found_it = false;
-
-	for (i = 0; i < connector->count_props; i++) {
-		struct drm_mode_get_property prop;
-
-		prop.prop_id = connector->props[i];
-		prop.count_values = 0;
-		prop.count_enum_blobs = 0;
-		if (drmIoctl(drm_fd, DRM_IOCTL_MODE_GETPROPERTY, &prop))
-			continue;
-
-		if (strcmp(prop.name, "DPMS"))
-			continue;
-
-		dpms = prop.prop_id;
-		found_it = true;
-		break;
-	}
-	igt_assert_f(found_it, "DPMS property not found on %d\n",
-		     connector->connector_id);
-
-	igt_assert(drmModeConnectorSetProperty(drm_fd, connector->connector_id,
-					       dpms, mode) == 0);
-}
-
 static void dpms_off_other_outputs(struct test_output *o)
 {
 	int i, n;
@@ -293,8 +319,10 @@ static void dpms_off_other_outputs(struct test_output *o)
 
 		connector = drmModeGetConnector(drm_fd, connector_id);
 
-		set_connector_dpms(connector,  DRM_MODE_DPMS_ON);
-		set_connector_dpms(connector,  DRM_MODE_DPMS_OFF);
+		kmstest_set_connector_dpms(drm_fd, connector,  DRM_MODE_DPMS_ON);
+		kmstest_set_connector_dpms(drm_fd, connector,  DRM_MODE_DPMS_OFF);
+
+		drmModeFreeConnector(connector);
 next:
 		;
 	}
@@ -303,7 +331,7 @@ next:
 static void set_dpms(struct test_output *o, int mode)
 {
 	for (int n = 0; n < o->count; n++)
-		set_connector_dpms(o->kconnector[n], mode);
+		kmstest_set_connector_dpms(drm_fd, o->kconnector[n], mode);
 }
 
 static void set_flag(unsigned int *v, unsigned int flag)
@@ -444,6 +472,33 @@ static double frame_time(struct test_output *o)
 	return 1000.0 * 1000.0 / o->kmode[0].vrefresh;
 }
 
+static void *vblank_wait_thread_func(void *data)
+{
+	struct test_output *o = data;
+	struct vblank_reply reply;
+	int i;
+
+	for (i = 0; i < 32; i++) {
+		unsigned long start = gettime_us();
+		__wait_for_vblank(TEST_VBLANK_BLOCK, o->pipe, 20, (unsigned long)o, &reply);
+		if (gettime_us() - start > 2 * frame_time(o))
+			return (void*)1;
+	}
+
+	return 0;
+}
+
+static void spawn_vblank_wait_thread(struct test_output *o)
+{
+	igt_assert(pthread_create(&vblank_wait_thread, NULL,
+				  vblank_wait_thread_func, o) == 0);
+}
+
+static void join_vblank_wait_thread(void)
+{
+	igt_assert(pthread_join(vblank_wait_thread, NULL) == 0);
+}
+
 static void fixup_premature_vblank_ts(struct test_output *o,
 				      struct event_state *es)
 {
@@ -484,54 +539,59 @@ static void check_state(struct test_output *o, struct event_state *es)
 	double usec_interflip;
 
 	timersub(&es->current_ts, &es->current_received_ts, &diff);
-	if ((!analog_tv_connector(o)) &&
-	    (diff.tv_sec > 0 || (diff.tv_sec == 0 && diff.tv_usec > 2000))) {
-		fprintf(stderr, "%s ts delayed for too long: %is, %iusec\n",
-			es->name, (int)diff.tv_sec, (int)diff.tv_usec);
-		igt_fail(5);
+	if (!analog_tv_connector(o)) {
+		igt_assert_f(diff.tv_sec < 0 || (diff.tv_sec == 0 && diff.tv_usec <= 2000),
+			     "%s ts delayed for too long: %is, %iusec\n",
+			     es->name, (int)diff.tv_sec, (int)diff.tv_usec);
+
 	}
 
 	if (es->count == 0)
 		return;
 
-	if (!timercmp(&es->last_received_ts, &es->current_ts, <)) {
-		fprintf(stderr, "%s ts before the %s was issued!\n",
-				es->name, es->name);
+	timersub(&es->current_ts, &es->last_received_ts, &diff);
+	igt_assert_f(timercmp(&es->last_received_ts, &es->current_ts, <),
+		     "%s ts before the %s was issued!\n"
+		     "timerdiff %is, %ius\n",
+		     es->name, es->name,
+		     (int) diff.tv_sec, (int) diff.tv_usec);
 
-		timersub(&es->current_ts, &es->last_received_ts, &diff);
-		fprintf(stderr, "timerdiff %is, %ius\n",
-			(int) diff.tv_sec, (int) diff.tv_usec);
-		igt_fail(6);
-	}
+	/* check only valid if no modeset happens in between, that increments by
+	 * (1 << 23) on each step. This bounding matches the one in
+	 * DRM_IOCTL_WAIT_VBLANK. */
+	if (!(o->flags & (TEST_DPMS | TEST_MODESET)))
+		igt_assert_f(es->current_seq - (es->last_seq + o->seq_step) <= 1UL << 23,
+			     "unexpected %s seq %u, should be >= %u\n",
+			     es->name, es->current_seq, es->last_seq + o->seq_step);
 
-	/* This bounding matches the one in DRM_IOCTL_WAIT_VBLANK. */
-	if (!(o->flags & (TEST_DPMS | TEST_MODESET))) {
-		/* check only valid if no modeset happens in between, that
-		 * increments by (1 << 23) on each step. */
-		if (es->current_seq - (es->last_seq + o->seq_step) > 1UL << 23) {
-			fprintf(stderr, "unexpected %s seq %u, should be >= %u\n",
-				es->name, es->current_seq, es->last_seq + o->seq_step);
-			igt_fail(10);
-		}
+	/* Check that the vblank frame didn't wrap unexpectedly. */
+	if (o->flags & TEST_TS_CONT) {
+		/* Ignore seq_step here since vblank waits time out immediately
+		 * when we kill the crtc. */
+		igt_assert_f(es->current_seq - es->last_seq >= 0,
+			     "unexpected %s seq %u, should be >= %u\n",
+			     es->name, es->current_seq, es->last_seq);
+		igt_assert_f(es->current_seq - es->last_seq <= 100,
+			     "unexpected %s seq %u, should be < %u\n",
+			     es->name, es->current_seq, es->last_seq + 100);
+
+		igt_debug("testing ts continuity: Current frame %u, old frame %u\n",
+			  es->current_seq, es->last_seq);
 	}
 
 	if ((o->flags & TEST_CHECK_TS) && (!analog_tv_connector(o))) {
 		timersub(&es->current_ts, &es->last_ts, &diff);
 		usec_interflip = (double)o->seq_step * frame_time(o);
-		if (fabs((((double) diff.tv_usec) - usec_interflip) /
-		    usec_interflip) > 0.005) {
-			fprintf(stderr, "inter-%s ts jitter: %is, %ius\n",
-				es->name,
-				(int) diff.tv_sec, (int) diff.tv_usec);
-			igt_fail(9);
-		}
 
-		if (es->current_seq != es->last_seq + o->seq_step) {
-			fprintf(stderr, "unexpected %s seq %u, expected %u\n",
-					es->name, es->current_seq,
-					es->last_seq + o->seq_step);
-			igt_fail(9);
-		}
+		igt_assert_f(fabs((((double) diff.tv_usec) - usec_interflip) /
+				  usec_interflip) <= 0.005,
+			     "inter-%s ts jitter: %is, %ius\n",
+			     es->name, (int) diff.tv_sec, (int) diff.tv_usec);
+
+		igt_assert_f(es->current_seq == es->last_seq + o->seq_step,
+			     "unexpected %s seq %u, expected %u\n",
+			     es->name, es->current_seq,
+			     es->last_seq + o->seq_step);
 	}
 }
 
@@ -580,7 +640,7 @@ static void check_all_state(struct test_output *o,
 static void recreate_fb(struct test_output *o)
 {
 	drmModeFBPtr r;
-	struct kmstest_fb *fb_info = &o->fb_info[o->current_fb_id];
+	struct igt_fb *fb_info = &o->fb_info[o->current_fb_id];
 	uint32_t new_fb_id;
 
 	/* Call rmfb/getfb/addfb to ensure those don't introduce stalls */
@@ -602,7 +662,7 @@ static void recreate_fb(struct test_output *o)
 static void set_y_tiling(struct test_output *o, int fb_idx)
 {
 	drmModeFBPtr r;
-	struct kmstest_fb *fb_info = &o->fb_info[fb_idx];
+	struct igt_fb *fb_info = &o->fb_info[fb_idx];
 
 	/* Call rmfb/getfb/addfb to ensure those don't introduce stalls */
 	r = drmModeGetFB(drm_fd, fb_info->fb_id);
@@ -613,48 +673,19 @@ static void set_y_tiling(struct test_output *o, int fb_idx)
 	drmFree(r);
 }
 
-
-static void exec_nop(int fd, uint32_t handle)
+static void stop_rings(bool stop)
 {
-	struct drm_i915_gem_execbuffer2 execbuf;
-	struct drm_i915_gem_exec_object2 gem_exec[1];
-	uint32_t b[2] = {MI_BATCH_BUFFER_END};
-
-	gem_write(fd, handle, 0, b, sizeof(b));
-
-	gem_exec[0].handle = handle;
-	gem_exec[0].relocation_count = 0;
-	gem_exec[0].relocs_ptr = 0;
-	gem_exec[0].alignment = 0;
-	gem_exec[0].offset = 0;
-	gem_exec[0].flags = 0;
-	gem_exec[0].rsvd1 = 0;
-	gem_exec[0].rsvd2 = 0;
-
-	execbuf.buffers_ptr = (uintptr_t)gem_exec;
-	execbuf.buffer_count = 1;
-	execbuf.batch_start_offset = 0;
-	execbuf.batch_len = 8;
-	execbuf.cliprects_ptr = 0;
-	execbuf.num_cliprects = 0;
-	execbuf.DR1 = 0;
-	execbuf.DR4 = 0;
-	execbuf.flags =  I915_EXEC_RENDER;
-	i915_execbuffer2_set_context_id(execbuf, 0);
-	execbuf.rsvd2 = 0;
-
-	igt_assert(drmIoctl(fd,
-			    DRM_IOCTL_I915_GEM_EXECBUFFER2,
-			    &execbuf) == 0);
+	if (stop)
+		igt_set_stop_rings(STOP_RING_DEFAULTS);
+	else
+		igt_set_stop_rings(STOP_RING_NONE);
 }
 
-static void eat_error_state(struct test_output *o)
+static void eat_error_state(void)
 {
 	static const char dfs_base[] = "/sys/kernel/debug/dri";
 	static const char dfs_entry_error[] = "i915_error_state";
-	static const char dfs_entry_stop[] = "i915_ring_stop";
 	static const char data[] = "";
-	static char tmp[128];
 	char fname[FILENAME_MAX];
 	int card_index = drm_get_card();
 	int fd;
@@ -673,38 +704,55 @@ static void eat_error_state(struct test_output *o)
 
 	/* and check whether stop_rings is not reset, i.e. the hang has indeed
 	 * happened */
-	snprintf(fname, FILENAME_MAX, "%s/%i/%s",
-		 dfs_base, card_index, dfs_entry_stop);
-
-	fd = open(fname, O_RDONLY);
-	igt_assert(fd >= 0);
-
-	igt_assert(read(fd, tmp, sizeof tmp) > 0);
-
-	igt_assert_f(atoi(tmp) == 0,
-		     "no gpu hang detected, stop_rings is still %s\n", tmp);
+	igt_assert_f(igt_get_stop_rings() == STOP_RING_NONE,
+		     "no gpu hang detected, stop_rings is still 0x%x\n",
+		     igt_get_stop_rings());
 
 	close(fd);
 }
 
-static void hang_gpu(struct test_output *o)
+static void unhang_gpu(int fd, uint32_t handle)
 {
-	static const char dfs_base[] = "/sys/kernel/debug/dri";
-	static const char dfs_entry[] = "i915_ring_stop";
-	static const char data[] = "0xf";
-	char fname[FILENAME_MAX];
-	int card_index = drm_get_card();
-	int fd;
+	gem_sync(drm_fd, handle);
+	gem_close(drm_fd, handle);
+	eat_error_state();
+	stop_rings(false);
+}
 
-	snprintf(fname, FILENAME_MAX, "%s/%i/%s",
-		 dfs_base, card_index, dfs_entry);
+static uint32_t hang_gpu(int fd)
+{
+	struct drm_i915_gem_execbuffer2 execbuf;
+	struct drm_i915_gem_exec_object2 gem_exec;
+	uint32_t b[2] = {MI_BATCH_BUFFER_END};
 
-	fd = open(fname, O_WRONLY);
-	igt_assert(fd >= 0);
+	stop_rings(true);
 
-	igt_assert(write(fd, data, sizeof(data)) == sizeof(data));
+	memset(&gem_exec, 0, sizeof(gem_exec));
+	gem_exec.handle = gem_create(fd, 4096);
+	gem_write(fd, gem_exec.handle, 0, b, sizeof(b));
 
-	close(fd);
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = (uintptr_t)&gem_exec;
+	execbuf.buffer_count = 1;
+	execbuf.batch_len = sizeof(b);
+
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &execbuf)) {
+		igt_assert_f(errno == EIO,
+			     "failed to exercise page flip hang recovery\n");
+
+		unhang_gpu(fd, gem_exec.handle);
+		gem_exec.handle = 0;
+	}
+
+	return gem_exec.handle;
+}
+
+static bool is_hung(int fd)
+{
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_THROTTLE, 0) == 0)
+		return false;
+
+	return errno == EIO;
 }
 
 static int set_mode(struct test_output *o, uint32_t fb, int x, int y)
@@ -741,7 +789,7 @@ static unsigned int run_test_step(struct test_output *o)
 	bool do_vblank;
 	struct vblank_reply vbl_reply;
 	unsigned int target_seq;
-	uint32_t handle = 0;	/* Suppress GCC warning */
+	uint32_t hang = 0;	/* Suppress GCC warning */
 
 	target_seq = o->vblank_state.seq_step;
 	/* Absolute waits only works once we have a frame counter. */
@@ -808,6 +856,15 @@ static unsigned int run_test_step(struct test_output *o)
 		igt_assert(do_wait_for_vblank(o, o->pipe, target_seq, &vbl_reply)
 		       == -EINVAL);
 
+	if (o->flags & TEST_VBLANK_RACE) {
+		spawn_vblank_wait_thread(o);
+
+		if (o->flags & TEST_MODESET)
+			igt_assert_f(set_mode(o, 0 /* no fb */, 0, 0) == 0,
+				     "failed to disable output: %s\n",
+				     strerror(errno));
+	}
+
 	if (o->flags & TEST_DPMS_OFF)
 		set_dpms(o, DRM_MODE_DPMS_OFF);
 
@@ -817,17 +874,38 @@ static unsigned int run_test_step(struct test_output *o)
 	if (o->flags & TEST_DPMS)
 		set_dpms(o, DRM_MODE_DPMS_ON);
 
-	printf("."); fflush(stdout);
+	if (o->flags & TEST_VBLANK_RACE) {
+		struct vblank_reply reply;
+		unsigned long start, end;
+
+		/* modeset/DPMS is done, vblank wait should work normally now */
+		start = gettime_us();
+		igt_assert(__wait_for_vblank(TEST_VBLANK_BLOCK, o->pipe, 1, 0, &reply) == 0);
+		end = gettime_us();
+		igt_assert(end - start > 1 * frame_time(o) / 2 &&
+			   end - start < 3 * frame_time(o) / 2);
+		join_vblank_wait_thread();
+	}
+
+	igt_info("."); fflush(stdout);
 
 	if (do_flip && (o->flags & TEST_HANG)) {
-		handle = gem_create(drm_fd, 4096);
-		hang_gpu(o);
-		exec_nop(drm_fd, handle);
+		hang = hang_gpu(drm_fd);
+		igt_assert_f(hang, "failed to exercise page flip hang recovery\n");
+	}
+
+	/* try to make sure we can issue two flips during the same frame */
+	if (do_flip && (o->flags & TEST_EBUSY)) {
+		struct vblank_reply reply;
+		igt_assert(__wait_for_vblank(TEST_VBLANK_BLOCK, o->pipe, 1, 0, &reply) == 0);
 	}
 
 	if (do_flip)
 		do_or_die(do_page_flip(o, new_fb_id, !(o->flags & TEST_NOEVENT)));
 
+	if (o->flags & TEST_FENCE_STRESS)
+		emit_fence_stress(o);
+
 	if (do_vblank) {
 		do_or_die(do_wait_for_vblank(o, o->pipe, target_seq,
 					     &vbl_reply));
@@ -851,6 +929,10 @@ static unsigned int run_test_step(struct test_output *o)
 			o->flip_state.count : o->vblank_state.count;
 		int x_ofs = count * 10 > o->fb_width - o->kmode[0].hdisplay ? o->fb_width - o->kmode[0].hdisplay : count * 10;
 
+		/* Make sure DSPSURF changes value */
+		if (o->flags & TEST_HANG)
+			o->current_fb_id = !o->current_fb_id;
+
 		igt_assert_f(set_mode(o, o->fb_ids[o->current_fb_id], x_ofs, 0) == 0,
 			     "failed to pan (%dx%d@%dHz)+%d: %s\n",
 			     o->kmode[0].hdisplay, o->kmode[0].vdisplay, o->kmode[0].vrefresh,
@@ -860,11 +942,17 @@ static unsigned int run_test_step(struct test_output *o)
 	if (o->flags & TEST_DPMS)
 		set_dpms(o, DRM_MODE_DPMS_OFF);
 
-	if (o->flags & TEST_MODESET && !(o->flags & TEST_RMFB))
+	if (o->flags & TEST_MODESET && !(o->flags & TEST_RMFB) && !(o->flags & TEST_VBLANK_RACE))
 		igt_assert_f(set_mode(o, 0 /* no fb */, 0, 0) == 0,
 			     "failed to disable output: %s\n",
 			     strerror(errno));
 
+	if (o->flags & TEST_RPM)
+		igt_assert(igt_wait_for_pm_status(IGT_RUNTIME_PM_STATUS_SUSPENDED));
+
+	if (o->flags & TEST_SUSPEND)
+		igt_system_suspend_autoresume();
+
 	if (do_vblank && (o->flags & TEST_EINVAL) && o->vblank_state.count > 0)
 		igt_assert(do_wait_for_vblank(o, o->pipe, target_seq, &vbl_reply)
 			   == -EINVAL);
@@ -872,11 +960,8 @@ static unsigned int run_test_step(struct test_output *o)
 	if (do_flip && (o->flags & TEST_EINVAL) && !(o->flags & TEST_FB_BAD_TILING))
 		igt_assert(do_page_flip(o, new_fb_id, true) == expected_einval);
 
-	if (do_flip && (o->flags & TEST_HANG)) {
-		gem_sync(drm_fd, handle);
-		gem_close(drm_fd, handle);
-		eat_error_state(o);
-	}
+	if (hang)
+		unhang_gpu(drm_fd, hang);
 
 	return completed_events;
 }
@@ -919,6 +1004,8 @@ static void connector_find_preferred_mode(uint32_t connector_id, int crtc_idx,
 
 	o->fb_width = o->kmode[0].hdisplay;
 	o->fb_height = o->kmode[0].vdisplay;
+
+	drmModeFreeCrtc(config.crtc);
 }
 
 static bool mode_compatible(const drmModeModeInfo *a, const drmModeModeInfo *b)
@@ -950,8 +1037,10 @@ static void connector_find_compatible_mode(int crtc_idx0, int crtc_idx1,
 		return;
 
 	if (kmstest_get_connector_config(drm_fd, o->_connector[1],
-					 1 << crtc_idx1, &config[1]) < 0)
+					 1 << crtc_idx1, &config[1]) < 0) {
+		kmstest_free_connector_config(&config[0]);
 		return;
+	}
 
 	mode[0] = &config[0].default_mode;
 	mode[1] = &config[1].default_mode;
@@ -984,15 +1073,18 @@ found:
 	o->kencoder[1] = config[1].encoder;
 	o->_crtc[1] = config[1].crtc->crtc_id;
 	o->kmode[1] = *mode[1];
+
+	drmModeFreeCrtc(config[0].crtc);
+	drmModeFreeCrtc(config[1].crtc);
 }
 
-static void paint_flip_mode(struct kmstest_fb *fb, bool odd_frame)
+static void paint_flip_mode(struct igt_fb *fb, bool odd_frame)
 {
-	cairo_t *cr = kmstest_get_cairo_ctx(drm_fd, fb);
+	cairo_t *cr = igt_get_cairo_ctx(drm_fd, fb);
 	int width = fb->width;
 	int height = fb->height;
 
-	kmstest_paint_test_pattern(cr, width, height);
+	igt_paint_test_pattern(cr, width, height);
 
 	if (odd_frame)
 		cairo_rectangle(cr, width/4, height/2, width/4, height/8);
@@ -1003,6 +1095,7 @@ static void paint_flip_mode(struct kmstest_fb *fb, bool odd_frame)
 	cairo_fill(cr);
 
 	igt_assert(!cairo_status(cr));
+	cairo_destroy(cr);
 }
 
 static int
@@ -1095,6 +1188,13 @@ static unsigned int wait_for_events(struct test_output *o)
 static unsigned event_loop(struct test_output *o, unsigned duration_ms)
 {
 	unsigned long start, end;
+	uint32_t hang = 0;	/* Suppress GCC warning */
+	int count = 0;
+
+	if (o->flags & TEST_HANG_ONCE) {
+		hang = hang_gpu(drm_fd);
+		igt_assert_f(hang, "failed to exercise page flip hang recovery\n");
+	}
 
 	start = gettime_us();
 
@@ -1107,12 +1207,17 @@ static unsigned event_loop(struct test_output *o, unsigned duration_ms)
 		check_all_state(o, completed_events);
 		update_all_state(o, completed_events);
 
-		if ((gettime_us() - start) / 1000 >= duration_ms)
+		if (count && (gettime_us() - start) / 1000 >= duration_ms)
 			break;
+
+		count++;
 	}
 
 	end = gettime_us();
 
+	if (hang)
+		unhang_gpu(drm_fd, hang);
+
 	/* Flush any remaining events */
 	if (o->pending_events)
 		wait_for_events(o);
@@ -1120,45 +1225,95 @@ static unsigned event_loop(struct test_output *o, unsigned duration_ms)
 	return end - start;
 }
 
-static void run_test_on_crtc(struct test_output *o, int crtc_idx, int duration_ms)
+static void free_test_output(struct test_output *o)
 {
-	unsigned elapsed;
+	int i;
 
-	connector_find_preferred_mode(o->_connector[0], crtc_idx, o);
+	for (i = 0; i < o->count; i++) {
+		drmModeFreeEncoder(o->kencoder[i]);
+		drmModeFreeConnector(o->kconnector[i]);
+	}
+}
+
+static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
+				 int crtc_count, int duration_ms)
+{
+	char test_name[128];
+	unsigned elapsed;
+	unsigned bo_size = 0;
+	bool tiled;
+	int i;
+
+	switch (crtc_count) {
+	case 1:
+		connector_find_preferred_mode(o->_connector[0], crtc_idxs[0], o);
+		snprintf(test_name, sizeof(test_name),
+			  "%s on crtc %d, connector %d",
+			  igt_subtest_name(), o->_crtc[0], o->_connector[0]);
+		break;
+	case 2:
+		connector_find_compatible_mode(crtc_idxs[0], crtc_idxs[1], o);
+		snprintf(test_name, sizeof(test_name),
+			 "%s on crtc %d:%d, connector %d:%d",
+			 igt_subtest_name(), o->_crtc[0], o->_crtc[1],
+			 o->_connector[0], o->_connector[1]);
+		break;
+	default:
+		igt_assert(0);
+	}
 	if (!o->mode_valid)
 		return;
 
+	igt_assert(o->count == crtc_count);
+
 	last_connector = o->kconnector[0];
 
-	fprintf(stdout, "Beginning %s on crtc %d, connector %d\n",
-		igt_subtest_name(), o->_crtc[0], o->_connector[0]);
+	igt_info("Beginning %s\n", test_name);
 
 	if (o->flags & TEST_PAN)
-		o->fb_width <<= 1;
+		o->fb_width *= 2;
 
-	o->fb_ids[0] = kmstest_create_fb(drm_fd, o->fb_width, o->fb_height,
-					 o->bpp, o->depth, false, &o->fb_info[0]);
-	o->fb_ids[1] = kmstest_create_fb(drm_fd, o->fb_width, o->fb_height,
-					 o->bpp, o->depth, false, &o->fb_info[1]);
-	o->fb_ids[2] = kmstest_create_fb(drm_fd, o->fb_width, o->fb_height,
-					 o->bpp, o->depth, true, &o->fb_info[2]);
+	tiled = false;
+	if (o->flags & TEST_FENCE_STRESS)
+		tiled = true;
+
+	/* 256 MB is usually the maximum mappable aperture,
+	 * (make it 4x times that to ensure failure) */
+	if (o->flags & TEST_BO_TOOBIG)
+		bo_size = 4*256*1024*1024;
+
+	o->fb_ids[0] = igt_create_fb(drm_fd, o->fb_width, o->fb_height,
+					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
+					 tiled, &o->fb_info[0]);
+	o->fb_ids[1] = igt_create_fb_with_bo_size(drm_fd, o->fb_width, o->fb_height,
+					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
+					 tiled, &o->fb_info[1], bo_size);
+	o->fb_ids[2] = igt_create_fb(drm_fd, o->fb_width, o->fb_height,
+					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
+					 true, &o->fb_info[2]);
 	igt_assert(o->fb_ids[0]);
 	igt_assert(o->fb_ids[1]);
-	igt_assert(o->fb_ids[2]);
+	if (o->flags & TEST_FB_BAD_TILING)
+		igt_require(o->fb_ids[2]);
 
 	paint_flip_mode(&o->fb_info[0], false);
-	paint_flip_mode(&o->fb_info[1], true);
-	paint_flip_mode(&o->fb_info[2], true);
+	if (!(o->flags & TEST_BO_TOOBIG))
+		paint_flip_mode(&o->fb_info[1], true);
+	if (o->fb_ids[2])
+		paint_flip_mode(&o->fb_info[2], true);
 
 	if (o->flags & TEST_FB_BAD_TILING)
 		set_y_tiling(o, 2);
 
-	kmstest_dump_mode(&o->kmode[0]);
+	for (i = 0; i < o->count; i++)
+		kmstest_dump_mode(&o->kmode[i]);
+
 	if (set_mode(o, o->fb_ids[0], 0, 0)) {
 		/* We may fail to apply the mode if there are hidden
 		 * constraints, such as bandwidth on the third pipe.
 		 */
-		igt_assert_f(crtc_idx < 2, "set_mode may only fail on the 3rd pipe\n");
+		igt_assert_f(crtc_count > 1 || crtc_idxs[0] < 2,
+			     "set_mode may only fail on the 3rd pipe or in multiple crtc tests\n");
 		goto out;
 	}
 	igt_assert(fb_is_bound(o, o->fb_ids[0]));
@@ -1167,10 +1322,15 @@ static void run_test_on_crtc(struct test_output *o, int crtc_idx, int duration_m
 	if (o->flags & TEST_CHECK_TS)
 		sleep(1);
 
-	igt_assert(do_page_flip(o, o->fb_ids[1], true) == 0);
+	if (o->flags & TEST_BO_TOOBIG) {
+		igt_assert(do_page_flip(o, o->fb_ids[1], true) == -E2BIG);
+		goto out;
+	} else
+		igt_assert(do_page_flip(o, o->fb_ids[1], true) == 0);
 	wait_for_events(o);
 
 	o->current_fb_id = 1;
+
 	if (o->flags & TEST_FLIP)
 		o->flip_state.seq_step = 1;
 	else
@@ -1190,105 +1350,17 @@ static void run_test_on_crtc(struct test_output *o, int crtc_idx, int duration_m
 	if (o->flags & TEST_VBLANK)
 		check_final_state(o, &o->vblank_state, elapsed);
 
-	fprintf(stdout, "\n%s on crtc %d, connector %d: PASSED\n\n",
-		igt_subtest_name(), o->_crtc[0], o->_connector[0]);
+	igt_info("\n%s: PASSED\n\n", test_name);
 
 out:
-	kmstest_remove_fb(drm_fd, &o->fb_info[2]);
-	kmstest_remove_fb(drm_fd, &o->fb_info[1]);
-	kmstest_remove_fb(drm_fd, &o->fb_info[0]);
+	if (o->fb_ids[2])
+		igt_remove_fb(drm_fd, &o->fb_info[2]);
+	igt_remove_fb(drm_fd, &o->fb_info[1]);
+	igt_remove_fb(drm_fd, &o->fb_info[0]);
 
 	last_connector = NULL;
 
-	drmModeFreeEncoder(o->kencoder[0]);
-	drmModeFreeConnector(o->kconnector[0]);
-}
-
-static void run_test_on_crtc_pair(struct test_output *o,
-				  int crtc_idx0, int crtc_idx1, int duration_ms)
-{
-	unsigned elapsed;
-	int i;
-
-	connector_find_compatible_mode(crtc_idx0, crtc_idx1, o);
-	if (!o->mode_valid)
-		return;
-
-	last_connector = o->kconnector[0];
-
-	fprintf(stdout, "Beginning %s on crtc %d:%d, connector %d:%d\n",
-		igt_subtest_name(), o->_crtc[0], o->_crtc[1], o->_connector[0], o->_connector[1]);
-
-	if (o->flags & TEST_PAN)
-		o->fb_width *= 2;
-
-	o->fb_ids[0] = kmstest_create_fb(drm_fd, o->fb_width, o->fb_height,
-					 o->bpp, o->depth, false, &o->fb_info[0]);
-	o->fb_ids[1] = kmstest_create_fb(drm_fd, o->fb_width, o->fb_height,
-					 o->bpp, o->depth, false, &o->fb_info[1]);
-	o->fb_ids[2] = kmstest_create_fb(drm_fd, o->fb_width, o->fb_height,
-					 o->bpp, o->depth, true, &o->fb_info[2]);
-	igt_assert(o->fb_ids[0]);
-	igt_assert(o->fb_ids[1]);
-	igt_assert(o->fb_ids[2]);
-
-	paint_flip_mode(&o->fb_info[0], false);
-	paint_flip_mode(&o->fb_info[1], true);
-	paint_flip_mode(&o->fb_info[2], true);
-
-	if (o->flags & TEST_FB_BAD_TILING)
-		set_y_tiling(o, 2);
-
-	kmstest_dump_mode(&o->kmode[0]);
-	kmstest_dump_mode(&o->kmode[1]);
-	if (set_mode(o, o->fb_ids[0], 0, 0)) {
-		/* We may fail to apply the mode if there are hidden
-		 * constraints, such as bandwidth on the third pipe.
-		 */
-		if (0) {
-			fprintf(stderr, "failed to set mode (%dx%d@%dHz): %s\n",
-				o->kmode[0].hdisplay, o->kmode[0].vdisplay, o->kmode[0].vrefresh,
-				strerror(errno));
-		}
-		goto out;
-	}
-	igt_assert(fb_is_bound(o, o->fb_ids[0]));
-
-	/* quiescent the hw a bit so ensure we don't miss a single frame */
-	if (o->flags & TEST_CHECK_TS)
-		sleep(1);
-
-	igt_assert(do_page_flip(o, o->fb_ids[1], true) == 0);
-	wait_for_events(o);
-
-	o->current_fb_id = 1;
-	o->flip_state.seq_step = 1;
-	if (o->flags & TEST_VBLANK_ABSOLUTE)
-		o->vblank_state.seq_step = 5;
-	else
-		o->vblank_state.seq_step = 1;
-
-	elapsed = event_loop(o, duration_ms);
-
-	if (o->flags & TEST_FLIP && !(o->flags & TEST_NOEVENT))
-		check_final_state(o, &o->flip_state, elapsed);
-	if (o->flags & TEST_VBLANK)
-		check_final_state(o, &o->vblank_state, elapsed);
-
-	fprintf(stdout, "\n%s on crtc %d:%d, connector %d:%d: PASSED\n\n",
-		igt_subtest_name(), o->_crtc[0], o->_crtc[1], o->_connector[0], o->_connector[1]);
-
-out:
-	kmstest_remove_fb(drm_fd, &o->fb_info[2]);
-	kmstest_remove_fb(drm_fd, &o->fb_info[1]);
-	kmstest_remove_fb(drm_fd, &o->fb_info[0]);
-
-	last_connector = NULL;
-
-	for (i = 0; i < o->count; i++) {
-		drmModeFreeEncoder(o->kencoder[i]);
-		drmModeFreeConnector(o->kconnector[i]);
-	}
+	free_test_output(o);
 }
 
 static int run_test(int duration, int flags)
@@ -1296,6 +1368,12 @@ static int run_test(int duration, int flags)
 	struct test_output o;
 	int i, n, modes = 0;
 
+	igt_require((flags & TEST_HANG) == 0 || !is_hung(drm_fd));
+
+
+	if (flags & TEST_RPM)
+		igt_require(igt_setup_runtime_pm());
+
 	resources = drmModeGetResources(drm_fd);
 	igt_assert(resources);
 
@@ -1315,6 +1393,7 @@ static int run_test(int duration, int flags)
 			if (o.mode_valid)
 				modes++;
 
+			free_test_output(&o);
 		}
 	}
 
@@ -1325,6 +1404,8 @@ static int run_test(int duration, int flags)
 	/* Find any connected displays */
 	for (i = 0; i < resources->count_connectors; i++) {
 		for (n = 0; n < resources->count_crtcs; n++) {
+			int crtc_idx;
+
 			memset(&o, 0, sizeof(o));
 			o.count = 1;
 			o._connector[0] = resources->connectors[i];
@@ -1334,7 +1415,8 @@ static int run_test(int duration, int flags)
 			o.bpp = 32;
 			o.depth = 24;
 
-			run_test_on_crtc(&o, n, duration);
+			crtc_idx = n;
+			run_test_on_crtc_set(&o, &crtc_idx, 1, duration);
 		}
 	}
 
@@ -1347,6 +1429,8 @@ static int run_pair(int duration, int flags)
 	struct test_output o;
 	int i, j, m, n, modes = 0;
 
+	igt_require((flags & TEST_HANG) == 0 || !is_hung(drm_fd));
+
 	resources = drmModeGetResources(drm_fd);
 	igt_assert(resources);
 
@@ -1369,6 +1453,7 @@ static int run_pair(int duration, int flags)
 					if (o.mode_valid)
 						modes++;
 
+					free_test_output(&o);
 				}
 			}
 		}
@@ -1385,6 +1470,8 @@ static int run_pair(int duration, int flags)
 		for (n = 0; n < resources->count_crtcs; n++) {
 			for (j = i + 1; j < resources->count_connectors; j++) {
 				for (m = n + 1; m < resources->count_crtcs; m++) {
+					int crtc_idxs[2];
+
 					memset(&o, 0, sizeof(o));
 					o.count = 2;
 					o._connector[0] = resources->connectors[i];
@@ -1395,7 +1482,11 @@ static int run_pair(int duration, int flags)
 					o.bpp = 32;
 					o.depth = 24;
 
-					run_test_on_crtc_pair(&o, n, m, duration);
+					crtc_idxs[0] = n;
+					crtc_idxs[1] = m;
+
+					run_test_on_crtc_set(&o, crtc_idxs, 2,
+							     duration);
 				}
 			}
 		}
@@ -1413,15 +1504,15 @@ static void get_timestamp_format(void)
 	ret = drmGetCap(drm_fd, DRM_CAP_TIMESTAMP_MONOTONIC, &cap_mono);
 	igt_assert(ret == 0 || errno == EINVAL);
 	monotonic_timestamp = ret == 0 && cap_mono == 1;
-	printf("Using %s timestamps\n",
-		monotonic_timestamp ? "monotonic" : "real");
+	igt_info("Using %s timestamps\n",
+		 monotonic_timestamp ? "monotonic" : "real");
 }
 
 static void kms_flip_exit_handler(int sig)
 {
 	igt_fixture {
 		if (last_connector)
-			set_connector_dpms(last_connector, DRM_MODE_DPMS_ON);
+			kmstest_set_connector_dpms(drm_fd, last_connector, DRM_MODE_DPMS_ON);
 	}
 }
 
@@ -1451,11 +1542,13 @@ int main(int argc, char **argv)
 		{ 60,  TEST_VBLANK | TEST_MODESET | TEST_WITH_DUMMY_RCS,
 					"rcs-wf_vblank-vs-modeset" },
 
-		{ 30, TEST_FLIP | TEST_EBUSY , "plain-flip" },
-		{ 30, TEST_FLIP | TEST_CHECK_TS | TEST_EBUSY , "plain-flip-ts-check" },
-		{ 30, TEST_FLIP | TEST_CHECK_TS | TEST_EBUSY | TEST_FB_RECREATE,
+		{ 30, TEST_FLIP , "plain-flip" },
+		{ 30, TEST_FLIP | TEST_EBUSY , "busy-flip" },
+		{ 30, TEST_FLIP | TEST_FENCE_STRESS , "flip-vs-fences" },
+		{ 30, TEST_FLIP | TEST_CHECK_TS, "plain-flip-ts-check" },
+		{ 30, TEST_FLIP | TEST_CHECK_TS | TEST_FB_RECREATE,
 			"plain-flip-fb-recreate" },
-		{ 30, TEST_FLIP | TEST_EBUSY | TEST_RMFB | TEST_MODESET , "flip-vs-rmfb" },
+		{ 30, TEST_FLIP | TEST_RMFB | TEST_MODESET , "flip-vs-rmfb" },
 		{ 60, TEST_FLIP | TEST_DPMS | TEST_EINVAL, "flip-vs-dpms" },
 		{ 60, TEST_FLIP | TEST_DPMS | TEST_WITH_DUMMY_BCS, "bcs-flip-vs-dpms" },
 		{ 60, TEST_FLIP | TEST_DPMS | TEST_WITH_DUMMY_RCS, "rcs-flip-vs-dpms" },
@@ -1476,6 +1569,7 @@ int main(int argc, char **argv)
 			TEST_CHECK_TS, "flip-vs-blocking-wf-vblank" },
 		{ 30, TEST_FLIP | TEST_MODESET | TEST_HANG | TEST_NOEVENT, "flip-vs-modeset-vs-hang" },
 		{ 30, TEST_FLIP | TEST_PAN | TEST_HANG, "flip-vs-panning-vs-hang" },
+		{ 30, TEST_VBLANK | TEST_HANG_ONCE, "vblank-vs-hang" },
 		{ 1, TEST_FLIP | TEST_EINVAL | TEST_FB_BAD_TILING, "flip-vs-bad-tiling" },
 
 		{ 1, TEST_DPMS_OFF | TEST_MODESET | TEST_FLIP,
@@ -1484,6 +1578,14 @@ int main(int argc, char **argv)
 					"single-buffer-flip-vs-dpms-off-vs-modeset" },
 		{ 30, TEST_FLIP | TEST_NO_2X_OUTPUT | TEST_DPMS_OFF_OTHERS , "dpms-off-confusion" },
 		{ 0, TEST_ENOENT | TEST_NOEVENT, "nonexisting-fb" },
+		{ 10, TEST_DPMS_OFF | TEST_DPMS | TEST_VBLANK_RACE, "dpms-vs-vblank-race" },
+		{ 10, TEST_MODESET | TEST_VBLANK_RACE, "modeset-vs-vblank-race" },
+		{ 10, TEST_VBLANK | TEST_DPMS | TEST_RPM | TEST_TS_CONT, "vblank-vs-dpms-rpm" },
+		{ 10, TEST_VBLANK | TEST_MODESET | TEST_RPM | TEST_TS_CONT, "vblank-vs-modeset-rpm" },
+		{ 0, TEST_VBLANK | TEST_DPMS | TEST_SUSPEND | TEST_TS_CONT, "vblank-vs-dpms-suspend" },
+		{ 0, TEST_VBLANK | TEST_MODESET | TEST_SUSPEND | TEST_TS_CONT, "vblank-vs-modeset-suspend" },
+		{ 0, TEST_VBLANK | TEST_SUSPEND | TEST_TS_CONT, "vblank-vs-suspend" },
+		{ 0, TEST_BO_TOOBIG | TEST_NO_2X_OUTPUT, "bo-too-big" },
 	};
 	int i;
 
@@ -1509,6 +1611,10 @@ int main(int argc, char **argv)
 		if (tests[i].flags & TEST_NO_2X_OUTPUT)
 			continue;
 
+		/* code doesn't disable all crtcs, so skip rpm tests */
+		if (tests[i].flags & TEST_RPM)
+			continue;
+
 		igt_subtest_f( "2x-%s", tests[i].name)
 			run_pair(tests[i].duration, tests[i].flags);
 	}
@@ -1527,13 +1633,19 @@ int main(int argc, char **argv)
 		if (tests[i].flags & TEST_NO_2X_OUTPUT)
 			continue;
 
+		/* code doesn't disable all crtcs, so skip rpm tests */
+		if (tests[i].flags & TEST_RPM)
+			continue;
+
 		igt_subtest_f( "2x-%s-interruptible", tests[i].name)
 			run_pair(tests[i].duration, tests[i].flags);
 	}
 	igt_stop_signal_helper();
 
-	igt_fixture
-		close(drm_fd);
+	/*
+	 * Let drm_fd leak, since it's needed by the dpms restore
+	 * exit_handler and igt_exit() won't return.
+	 */
 
 	igt_exit();
 }
