@@ -30,20 +30,70 @@
 #include "drmtest.h"
 #include "igt_debugfs.h"
 #include "igt_kms.h"
+#include "igt_aux.h"
+#include "ioctl_wrappers.h"
 
 typedef struct {
 	int drm_fd;
-	igt_debugfs_t debugfs;
 	igt_display_t display;
-	struct kmstest_fb fb;
+	struct igt_fb fb;
 } data_t;
 
+static uint64_t submit_batch(int fd, unsigned ring_id)
+{
+	const uint32_t batch[] = { MI_NOOP,
+				   MI_BATCH_BUFFER_END };
+	struct drm_i915_gem_execbuffer2 execbuf;
+	struct drm_i915_gem_exec_object2 exec;
+	uint64_t presumed_offset;
+
+	gem_require_ring(fd, ring_id);
+
+	exec.handle = gem_create(fd, 4096);
+	gem_write(fd, exec.handle, 0, batch, sizeof(batch));
+	exec.relocation_count = 0;
+	exec.relocs_ptr = 0;
+	exec.alignment = 0;
+	exec.offset = 0;
+	exec.flags = 0;
+	exec.rsvd1 = 0;
+	exec.rsvd2 = 0;
+
+	execbuf.buffers_ptr = (uintptr_t)&exec;
+	execbuf.buffer_count = 1;
+	execbuf.batch_start_offset = 0;
+	execbuf.batch_len = sizeof(batch);
+	execbuf.cliprects_ptr = 0;
+	execbuf.num_cliprects = 0;
+	execbuf.DR1 = 0;
+	execbuf.DR4 = 0;
+	execbuf.flags = ring_id;
+	i915_execbuffer2_set_context_id(execbuf, 0);
+	execbuf.rsvd2 = 0;
+
+	gem_execbuf(fd, &execbuf);
+	gem_sync(fd, exec.handle);
+	presumed_offset = exec.offset;
+
+	igt_set_stop_rings(igt_to_stop_ring_flag(ring_id));
+
+	gem_execbuf(fd, &execbuf);
+	gem_sync(fd, exec.handle);
+
+	igt_assert(igt_get_stop_rings() == STOP_RING_NONE);
+	igt_assert(presumed_offset == exec.offset);
+
+	gem_close(fd, exec.handle);
+
+	return exec.offset;
+}
+
 static void test_bad_command(data_t *data, const char *cmd)
 {
 	FILE *ctl;
 	size_t written;
 
-	ctl = igt_debugfs_fopen(&data->debugfs, "i915_display_crc_ctl", "r+");
+	ctl = igt_debugfs_fopen("i915_display_crc_ctl", "r+");
 	written = fwrite(cmd, 1, strlen(cmd), ctl);
 	fflush(ctl);
 	igt_assert_cmpint(written, ==, (strlen(cmd)));
@@ -71,12 +121,12 @@ static void test_read_crc(data_t *data, int pipe, unsigned flags)
 
 		igt_output_set_pipe(output, pipe);
 
-		fprintf(stdout, "%s: Testing connector %s using pipe %c\n",
-			igt_subtest_name(), igt_output_name(output),
-			pipe_name(pipe));
+		igt_info("%s: Testing connector %s using pipe %c\n",
+			 igt_subtest_name(), igt_output_name(output),
+			 pipe_name(pipe));
 
 		mode = igt_output_get_mode(output);
-		kmstest_create_color_fb(data->drm_fd,
+		igt_create_color_fb(data->drm_fd,
 					mode->hdisplay, mode->vdisplay,
 					DRM_FORMAT_XRGB8888,
 					false, /* tiled */
@@ -88,8 +138,7 @@ static void test_read_crc(data_t *data, int pipe, unsigned flags)
 
 		igt_display_commit(display);
 
-		pipe_crc = igt_pipe_crc_new(&data->debugfs,
-					    pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
+		pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
 
 		if (!pipe_crc)
 			continue;
@@ -118,18 +167,20 @@ static void test_read_crc(data_t *data, int pipe, unsigned flags)
 
 		free(crcs);
 		igt_pipe_crc_free(pipe_crc);
-		kmstest_remove_fb(data->drm_fd, &data->fb);
+		igt_remove_fb(data->drm_fd, &data->fb);
 		igt_plane_set_fb(primary, NULL);
+
+		igt_output_set_pipe(output, PIPE_ANY);
 	}
 
 	igt_require_f(valid_connectors, "No connector found for pipe %i\n", pipe);
 
 }
 
+data_t data = {0, };
+
 igt_main
 {
-	data_t data = {0, };
-
 	igt_skip_on_simulation();
 
 	igt_fixture {
@@ -137,8 +188,7 @@ igt_main
 
 		igt_set_vt_graphics_mode();
 
-		igt_debugfs_init(&data.debugfs);
-		igt_pipe_crc_check(&data.debugfs);
+		igt_require_pipe_crc();
 
 		igt_display_init(&data.display, data.drm_fd);
 	}
@@ -161,6 +211,18 @@ igt_main
 
 		igt_subtest_f("read-crc-pipe-%c-frame-sequence", 'A'+i)
 			test_read_crc(&data, i, TEST_SEQUENCE);
+
+		igt_subtest_f("suspend-read-crc-pipe-%c", 'A'+i) {
+			igt_system_suspend_autoresume();
+
+			test_read_crc(&data, i, 0);
+		}
+
+		igt_subtest_f("hang-read-crc-pipe-%c", 'A'+i) {
+			submit_batch(data.drm_fd, I915_EXEC_RENDER);
+
+			test_read_crc(&data, i, 0);
+		}
 	}
 
 	igt_fixture {
