@@ -35,7 +35,6 @@
 
 typedef struct {
 	int drm_fd;
-	igt_debugfs_t debugfs;
 	igt_display_t display;
 } data_t;
 
@@ -64,22 +63,22 @@ static void
 create_fb_for_mode__position(data_t *data, drmModeModeInfo *mode,
 			     double rect_x, double rect_y,
 			     double rect_w, double rect_h,
-			     struct kmstest_fb *fb /* out */)
+			     struct igt_fb *fb /* out */)
 {
 	unsigned int fb_id;
 	cairo_t *cr;
 
-	fb_id = kmstest_create_fb(data->drm_fd,
+	fb_id = igt_create_fb(data->drm_fd,
 				  mode->hdisplay, mode->vdisplay,
-				  32 /* bpp */, 24 /* depth */,
+				  DRM_FORMAT_XRGB8888,
 				  false /* tiling */,
 				  fb);
 	igt_assert(fb_id);
 
-	cr = kmstest_get_cairo_ctx(data->drm_fd, fb);
-	kmstest_paint_color(cr, 0, 0, mode->hdisplay, mode->vdisplay,
+	cr = igt_get_cairo_ctx(data->drm_fd, fb);
+	igt_paint_color(cr, 0, 0, mode->hdisplay, mode->vdisplay,
 			    0.0, 1.0, 0.0);
-	kmstest_paint_color(cr, rect_x, rect_y, rect_w, rect_h, 0.0, 0.0, 0.0);
+	igt_paint_color(cr, rect_x, rect_y, rect_w, rect_h, 0.0, 0.0, 0.0);
 	igt_assert(cairo_status(cr) == 0);
 	cairo_destroy(cr);
 }
@@ -88,18 +87,17 @@ static void
 test_position_init(test_position_t *test, igt_output_t *output, enum pipe pipe)
 {
 	data_t *data = test->data;
-	struct kmstest_fb green_fb;
+	struct igt_fb green_fb;
 	drmModeModeInfo *mode;
 	igt_plane_t *primary;
 
-	test->pipe_crc = igt_pipe_crc_new(&data->debugfs,
-					  pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
+	test->pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
 
 	igt_output_set_pipe(output, pipe);
 	primary = igt_output_get_plane(output, 0);
 
 	mode = igt_output_get_mode(output);
-	kmstest_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
+	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 				DRM_FORMAT_XRGB8888,
 				false, /* tiled */
 				0.0, 1.0, 0.0,
@@ -113,7 +111,7 @@ test_position_init(test_position_t *test, igt_output_t *output, enum pipe pipe)
 	igt_plane_set_fb(primary, NULL);
 	igt_display_commit(&data->display);
 
-	kmstest_remove_fb(data->drm_fd, &green_fb);
+	igt_remove_fb(data->drm_fd, &green_fb);
 }
 
 static void
@@ -138,12 +136,12 @@ test_plane_position_with_output(data_t *data,
 {
 	test_position_t test = { .data = data };
 	igt_plane_t *primary, *sprite;
-	struct kmstest_fb primary_fb, sprite_fb;
+	struct igt_fb primary_fb, sprite_fb;
 	drmModeModeInfo *mode;
 	igt_crc_t crc;
 
-	fprintf(stdout, "Testing connector %s using pipe %c plane %d\n",
-		igt_output_name(output), pipe_name(pipe), plane);
+	igt_info("Testing connector %s using pipe %c plane %d\n",
+		 igt_output_name(output), pipe_name(pipe), plane);
 
 	test_position_init(&test, output, pipe);
 
@@ -155,7 +153,7 @@ test_plane_position_with_output(data_t *data,
 				     &primary_fb);
 	igt_plane_set_fb(primary, &primary_fb);
 
-	kmstest_create_color_fb(data->drm_fd,
+	igt_create_color_fb(data->drm_fd,
 				64, 64, /* width, height */
 				DRM_FORMAT_XRGB8888,
 				false, /* tiled */
@@ -231,8 +229,7 @@ igt_main
 
 		igt_set_vt_graphics_mode();
 
-		igt_debugfs_init(&data.debugfs);
-		igt_pipe_crc_check(&data.debugfs);
+		igt_require_pipe_crc();
 		igt_display_init(&data.display, data.drm_fd);
 	}
 
