@@ -44,7 +44,7 @@
 #include <pthread.h>
 
 #include "drm.h"
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 #include "igt_debugfs.h"
 
@@ -258,6 +258,11 @@ static void test_reimport_close_race(void)
 	int obj_count;
 	void *status;
 	uint32_t handle;
+	int fake;
+
+	/* Allocate exit handler fds in here so that we dont screw
+	 * up the counts */
+	fake = drm_open_any();
 
 	obj_count = get_object_count();
 
@@ -293,7 +298,10 @@ static void test_reimport_close_race(void)
 
 	obj_count = get_object_count() - obj_count;
 
-	printf("leaked %i objects\n", obj_count);
+	igt_info("leaked %i objects\n", obj_count);
+
+	close(fake);
+
 	igt_assert(obj_count == 0);
 }
 
@@ -367,7 +375,7 @@ static void test_export_close_race(void)
 
 	obj_count = get_object_count() - obj_count;
 
-	printf("leaked %i objects\n", obj_count);
+	igt_info("leaked %i objects\n", obj_count);
 	igt_assert(obj_count == 0);
 }
 
