@@ -56,13 +56,14 @@
 #include <stdbool.h>
 #include <strings.h>
 #include <unistd.h>
+#include <termios.h>
 #include <sys/poll.h>
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 #include "testdisplay.h"
 #include "igt_kms.h"
@@ -70,6 +71,9 @@
 #include <stdlib.h>
 #include <signal.h>
 
+static int tio_fd;
+struct termios saved_tio;
+
 drmModeRes *resources;
 int drm_fd, modes;
 int test_all_modes = 0, test_preferred_mode = 0, force_mode = 0, test_plane,
@@ -221,7 +225,7 @@ static void connector_find_preferred_mode(uint32_t connector_id,
 }
 
 static void
-paint_color_key(struct kmstest_fb *fb_info)
+paint_color_key(struct igt_fb *fb_info)
 {
 	int i, j;
 	uint32_t *fb_ptr;
@@ -272,9 +276,9 @@ static void paint_image(cairo_t *cr, const char *file)
 	cairo_surface_destroy(image);
 }
 
-static void paint_output_info(struct connector *c, struct kmstest_fb *fb)
+static void paint_output_info(struct connector *c, struct igt_fb *fb)
 {
-	cairo_t *cr = kmstest_get_cairo_ctx(drm_fd, fb);
+	cairo_t *cr = igt_get_cairo_ctx(drm_fd, fb);
 	int l_width = fb->width;
 	int l_height = fb->height;
 	double str_width;
@@ -282,20 +286,17 @@ static void paint_output_info(struct connector *c, struct kmstest_fb *fb)
 	double max_width;
 	int i;
 
-	kmstest_paint_test_pattern(cr, l_width, l_height);
+	igt_paint_test_pattern(cr, l_width, l_height);
 
-	cairo_select_font_face(cr, "Helvetica",
-			       CAIRO_FONT_SLANT_NORMAL,
-			       CAIRO_FONT_WEIGHT_NORMAL);
 	cairo_move_to(cr, l_width / 2, l_height / 2);
 
 	/* Print connector and mode name */
 	cairo_set_font_size(cr, 48);
-	kmstest_cairo_printf_line(cr, align_hcenter, 10, "%s",
+	igt_cairo_printf_line(cr, align_hcenter, 10, "%s",
 		 kmstest_connector_type_str(c->connector->connector_type));
 
 	cairo_set_font_size(cr, 36);
-	str_width = kmstest_cairo_printf_line(cr, align_hcenter, 10,
+	str_width = igt_cairo_printf_line(cr, align_hcenter, 10,
 		"%s @ %dHz on %s encoder", c->mode.name, c->mode.vrefresh,
 		kmstest_encoder_type_str(c->encoder->encoder_type));
 
@@ -303,7 +304,7 @@ static void paint_output_info(struct connector *c, struct kmstest_fb *fb)
 
 	/* List available modes */
 	cairo_set_font_size(cr, 18);
-	str_width = kmstest_cairo_printf_line(cr, align_left, 10,
+	str_width = igt_cairo_printf_line(cr, align_left, 10,
 					      "Available modes:");
 	cairo_rel_move_to(cr, str_width, 0);
 	cairo_get_current_point(cr, &x, &top_y);
@@ -316,7 +317,7 @@ static void paint_output_info(struct connector *c, struct kmstest_fb *fb)
 			max_width = 0;
 			cairo_move_to(cr, x, top_y);
 		}
-		str_width = kmstest_cairo_printf_line(cr, align_right, 10,
+		str_width = igt_cairo_printf_line(cr, align_right, 10,
 			"%s @ %dHz", c->connector->modes[i].name,
 			 c->connector->modes[i].vrefresh);
 		if (str_width > max_width)
@@ -346,39 +347,11 @@ static void set_single(void)
 		perror("Could not set signal handler");
 }
 
-static void set_connector_dpms(drmModeConnector *connector, int mode)
-{
-	int i, dpms = 0;
-	bool found_it = false;
-
-	for (i = 0; i < connector->count_props; i++) {
-		struct drm_mode_get_property prop;
-
-		prop.prop_id = connector->props[i];
-		prop.count_values = 0;
-		prop.count_enum_blobs = 0;
-		if (drmIoctl(drm_fd, DRM_IOCTL_MODE_GETPROPERTY, &prop))
-			continue;
-
-		if (strcmp(prop.name, "DPMS"))
-			continue;
-
-		dpms = prop.prop_id;
-		found_it = true;
-		break;
-	}
-	igt_assert_f(found_it, "DPMS property not found on %d\n",
-		     connector->connector_id);
-
-	igt_assert(drmModeConnectorSetProperty(drm_fd, connector->connector_id,
-					       dpms, mode) == 0);
-}
-
 static void
 set_mode(struct connector *c)
 {
 	unsigned int fb_id = 0;
-	struct kmstest_fb fb_info[2] = { };
+	struct igt_fb fb_info[2] = { };
 	int j, test_mode_num, current_fb = 0, old_fb = -1;
 
 	test_mode_num = 1;
@@ -405,7 +378,8 @@ set_mode(struct connector *c)
 		width = c->mode.hdisplay;
 		height = c->mode.vdisplay;
 
-		fb_id = kmstest_create_fb(drm_fd, width, height, bpp, depth,
+		fb_id = igt_create_fb(drm_fd, width, height,
+					  igt_bpp_depth_to_drm_format(bpp, depth),
 					  enable_tiling, &fb_info[current_fb]);
 		paint_output_info(c, &fb_info[current_fb]);
 		paint_color_key(&fb_info[current_fb]);
@@ -421,7 +395,7 @@ set_mode(struct connector *c)
 		}
 
 		if (old_fb != -1)
-			kmstest_remove_fb(drm_fd, &fb_info[old_fb]);
+			igt_remove_fb(drm_fd, &fb_info[old_fb]);
 		old_fb = current_fb;
 		current_fb = 1 - current_fb;
 
@@ -429,9 +403,9 @@ set_mode(struct connector *c)
 			sleep(sleep_between_modes);
 
 		if (do_dpms) {
-			set_connector_dpms(c->connector, do_dpms);
+			kmstest_set_connector_dpms(drm_fd, c->connector, do_dpms);
 			sleep(sleep_between_modes);
-			set_connector_dpms(c->connector, DRM_MODE_DPMS_ON);
+			kmstest_set_connector_dpms(drm_fd, c->connector, DRM_MODE_DPMS_ON);
 		}
 
 		if (qr_code){
@@ -441,7 +415,7 @@ set_mode(struct connector *c)
 	}
 
 	if (test_all_modes)
-		kmstest_remove_fb(drm_fd, &fb_info[old_fb]);
+		igt_remove_fb(drm_fd, &fb_info[old_fb]);
 
 	drmModeFreeEncoder(c->encoder);
 	drmModeFreeConnector(c->connector);
@@ -504,7 +478,7 @@ static void stereo_fb_layout_from_mode(struct stereo_fb_layout *layout,
 		break;
 	}
 	default:
-		assert(0);
+		igt_assert(0);
 	}
 }
 
@@ -520,25 +494,26 @@ static const char *stereo_mode_str(drmModeModeInfo *mode)
 	case DRM_MODE_FLAG_3D_FRAME_PACKING:
 		return "FP";
 	default:
-		assert(0);
+		igt_assert(0);
 	}
 }
 
-static uint32_t create_stereo_fb(drmModeModeInfo *mode, struct kmstest_fb *fb)
+static uint32_t create_stereo_fb(drmModeModeInfo *mode, struct igt_fb *fb)
 {
 	struct stereo_fb_layout layout;
 	cairo_t *cr;
 	uint32_t fb_id;
 
 	stereo_fb_layout_from_mode(&layout, mode);
-	fb_id = kmstest_create_fb(drm_fd, layout.fb_width, layout.fb_height,
-				  bpp, depth, enable_tiling, fb);
-	cr = kmstest_get_cairo_ctx(drm_fd, fb);
+	fb_id = igt_create_fb(drm_fd, layout.fb_width, layout.fb_height,
+				  igt_bpp_depth_to_drm_format(bpp, depth),
+				  enable_tiling, fb);
+	cr = igt_get_cairo_ctx(drm_fd, fb);
 
-	kmstest_paint_image(cr, IGT_DATADIR"/1080p-left.png",
+	igt_paint_image(cr, IGT_DATADIR"/1080p-left.png",
 			    layout.left.x, layout.left.y,
 			    layout.left.width, layout.left.height);
-	kmstest_paint_image(cr, IGT_DATADIR"/1080p-right.png",
+	igt_paint_image(cr, IGT_DATADIR"/1080p-right.png",
 			    layout.right.x, layout.right.y,
 			    layout.right.width, layout.right.height);
 
@@ -553,7 +528,7 @@ static uint32_t create_stereo_fb(drmModeModeInfo *mode, struct kmstest_fb *fb)
 			 mode->vrefresh,
 			 stereo_mode_str(mode));
 
-		kmstest_write_fb(drm_fd, fb, buffer);
+		igt_write_fb_to_png(drm_fd, fb, buffer);
 	}
 
 	return fb_id;
@@ -562,7 +537,7 @@ static uint32_t create_stereo_fb(drmModeModeInfo *mode, struct kmstest_fb *fb)
 static void do_set_stereo_mode(struct connector *c)
 {
 	uint32_t fb_id;
-	struct kmstest_fb fb_info;
+	struct igt_fb fb_info;
 
 	fb_id = create_stereo_fb(&c->mode, &fb_info);
 
@@ -606,9 +581,9 @@ set_stereo_mode(struct connector *c)
 			sleep(sleep_between_modes);
 
 		if (do_dpms) {
-			set_connector_dpms(c->connector, DRM_MODE_DPMS_OFF);
+			kmstest_set_connector_dpms(drm_fd, c->connector, DRM_MODE_DPMS_OFF);
 			sleep(sleep_between_modes);
-			set_connector_dpms(c->connector, DRM_MODE_DPMS_ON);
+			kmstest_set_connector_dpms(drm_fd, c->connector, DRM_MODE_DPMS_ON);
 		}
 	}
 
@@ -721,6 +696,12 @@ static void __attribute__((noreturn)) usage(char *name)
 
 #define dump_resource(res) if (res) dump_##res()
 
+static void cleanup_and_exit(int ret)
+{
+	close(drm_fd);
+	exit(ret);
+}
+
 static gboolean input_event(GIOChannel *source, GIOCondition condition,
 				gpointer data)
 {
@@ -729,7 +710,7 @@ static gboolean input_event(GIOChannel *source, GIOCondition condition,
 
 	count = read(g_io_channel_unix_get_fd(source), buf, sizeof(buf));
 	if (buf[0] == 'q' && (count == 1 || buf[1] == '\n')) {
-		exit(0);
+		cleanup_and_exit(0);
 	}
 
 	return TRUE;
@@ -755,6 +736,24 @@ static void enter_exec_path( char **argv )
 	free(exec_path);
 }
 
+static void restore_termio_mode(int sig)
+{
+	tcsetattr(tio_fd, TCSANOW, &saved_tio);
+	close(tio_fd);
+}
+
+static void set_termio_mode(void)
+{
+	struct termios tio;
+
+	tio_fd = dup(STDIN_FILENO);
+	tcgetattr(tio_fd, &saved_tio);
+	igt_install_exit_handler(restore_termio_mode);
+	tio = saved_tio;
+	tio.c_lflag &= ~(ICANON | ECHO);
+	tcsetattr(tio_fd, TCSANOW, &tio);
+}
+
 int main(int argc, char **argv)
 {
 	int c;
@@ -827,6 +826,8 @@ int main(int argc, char **argv)
 		}
 	}
 
+	set_termio_mode();
+
 	if (depth <= 8)
 		bpp = 8;
 	else if (depth <= 16)
